/*
* Copyright 2021-2022 Objectiv B.V.
*/
import { ApplicationContext, AbstractGlobalContext, AbstractLocationContext, Contexts, AbstractEvent, ContentContext, CookieIdContext, ExpandableContext, HttpContext, InputContext, LinkContext, MediaPlayerContext, NavigationContext, OverlayContext, PathContext, PressableContext, RootLocationContext, SessionContext, ApplicationLoadedEvent, FailureEvent, HiddenEvent, InputChangeEvent, InteractiveEvent, MediaEvent, MediaLoadEvent, MediaPauseEvent, MediaStartEvent, MediaStopEvent, NonInteractiveEvent, PressEvent, SuccessEvent, VisibleEvent } from "@objectiv/schema";
/*
* Copyright 2021-2022 Objectiv B.V.
*/
import ExtendableError from "es6-error";
/*
* Copyright 2021-2022 Objectiv B.V.
*/
import { Infer, Struct } from "superstruct";
/**
 * An array of Location Contexts
 */
type LocationStack = AbstractLocationContext[];
/**
 * An array of Global Contexts
 */
type GlobalContexts = AbstractGlobalContext[];
/**
 * The configuration of the Contexts interface
 */
type ContextsConfig = {
    location_stack?: LocationStack;
    global_contexts?: GlobalContexts;
};
/**
 * TrackerConsole is a simplified implementation of Console.
 */
type TrackerConsole = Pick<Console, "debug" | "error" | "group" | "groupCollapsed" | "groupEnd" | "info" | "log" | "warn">;
/**
 * NoopConsole is an empty implementation of TrackerConsole that doesn't do anything.
 * Its main purpose is to forcefully disable the Console during dev mode or testing.
 */
declare const NoopConsole: TrackerConsole;
/**
 * TrackerEvents are simply a combination of an `event` name and their Contexts.
 * Contexts are entirely optional, although Collectors will mostly likely enforce minimal requirements around them.
 * Eg. An interactive TrackerEvent without a Location Stack is probably not descriptive enough to be acceptable.
 */
type TrackerEventConfig = Pick<AbstractEvent, "_type"> & ContextsConfig & {
    // Unless the Event config has been preconfigured with an id the TrackerEvent will generate one for us
    id?: string;
};
/**
 * An Event before it has been handed over to the Tracker.
 * Properties that will be set by the Tracker or Transport are omitted: `id`, `time`
 */
type UntrackedEvent = Omit<AbstractEvent, "id" | "time">;
/**
 * Our main TrackerEvent interface and basic implementation
 */
declare class TrackerEvent implements UntrackedEvent, Contexts {
    // Event interface
    readonly _type: string;
    id: string;
    time?: number;
    // Contexts interface
    readonly location_stack: AbstractLocationContext[];
    readonly global_contexts: AbstractGlobalContext[];
    /**
     * Configures the TrackerEvent instance via a TrackerEventConfig and optionally one or more ContextConfig.
     *
     * TrackerEventConfig is used mainly to configure the `event` property, although it can also carry Contexts.
     *
     * ContextConfigs are used to configure location_stack and global_contexts. If multiple configurations have been
     * provided they will be merged onto each other to produce a single location_stack and global_contexts.
     */
    constructor({ _type, id, ...otherEventProps }: TrackerEventConfig, ...contextConfigs: ContextsConfig[]);
    /**
     * Tracking time setter.
     * Defaults to Date.now() if not timestampMs is provided.
     */
    setTime(timestampMs?: number): void;
    /**
     * Custom JSON serializer that cleans up the internally properties we use internally to differentiate between
     * Contexts and Event types and for validation. This ensures the Event we send to Collectors has only OSF properties.
     */
    toJSON(): this & {
        location_stack: ({} & AbstractLocationContext)[];
        global_contexts: ({} & AbstractGlobalContext)[];
    };
}
/**
 * All possible lifecycle methods of a TrackerPlugin.
 */
interface TrackerPluginLifecycleInterface {
    /**
     * Executed when the Tracker initializes.
     * Useful to factor Contexts that will not change during this tracking session.
     */
    initialize?: (contexts: Required<ContextsConfig>) => void;
    /**
     * Executed before the TrackerEvent is handed over to the TrackerTransport.
     * Useful to factor Contexts that may have changed from the last TrackerEvent tracking. Eg: URL, Time, User, etc
     */
    beforeTransport?: (contexts: Required<ContextsConfig>) => void;
}
/**
 * The TrackerPluginConfig.
 */
type TrackerPluginConfig = {
    /**
     * Optional. A TrackerConsole instance for logging.
     */
    console?: TrackerConsole;
};
/**
 * A TrackerPlugin must define its own `pluginName` and may define TrackerPluginLifecycle callbacks.
 * It also defines a method to determine if the plugin can be used. Similarly to the Transport interface, this can
 * be used to check environment requirements, APIs availability, etc.
 */
interface TrackerPluginInterface extends TrackerPluginLifecycleInterface {
    readonly console?: TrackerConsole;
    readonly pluginName: string;
    /**
     * Should return if the TrackerPlugin can be used. Eg: a browser based plugin may want to return `false` during SSR.
     */
    isUsable(): boolean;
}
/**
 * The TrackerPlugin constructor interface.
 */
interface TrackerPluginConstructor {
    new (pluginConfig: TrackerPluginConfig): TrackerPluginInterface;
}
/**
 * TrackerPlugins can be specified by instance, name or even on the fly as Objects.
 *
 * @example
 *
 *  Given a hypothetical PluginA:
 *
 *    class PluginA implements TrackerPlugin {
 *      readonly pluginName = 'pluginA';
 *      readonly parameter?: string;
 *
 *      constructor(args?: { parameter?: string }) {
 *        this.parameter = args?.parameter;
 *      }
 *
 *      isUsable() {
 *        return true;
 *      }
 *    }
 *
 *  And its factory:
 *
 *    const PluginAFactory = (parameter: string) => new PluginA({ parameter });
 *
 *  These would be all valid ways of adding it to the Plugins list:
 *
 *    PluginA
 *    new PluginA()
 *    new PluginA({ parameter: 'parameterValue' })
 *    PluginAFactory('parameterValue')
 *
 *  And it's also possible to define a Plugin on the fly as an Object:
 *
 *    {
 *      pluginName: 'pluginA',
 *      parameter: 'parameterValue',
 *      isUsable: () => true
 *    } as TrackerPlugin
 *
 */
type TrackerPluginsConfiguration = TrackerPluginConfig & {
    plugins: TrackerPluginInterface[];
};
/**
 * TrackerPlugins is responsible for constructing TrackerPlugin instances and orchestrating their callbacks.
 * It also makes sure to check if Plugins are usable, before executing their callbacks.
 *
 * @note plugin order matters, as they are executed sequentially, a plugin executed later has access to previous
 * Plugins mutations. For example a plugin meant to access the finalized version of the TrackerEvent should be placed
 * at the bottom of the list.
 */
declare class TrackerPlugins implements TrackerPluginLifecycleInterface {
    readonly console?: TrackerConsole;
    readonly plugins: TrackerPluginInterface[];
    /**
     * Plugins can be lazy. Map through them to instantiate them.
     */
    constructor(trackerPluginsConfig: TrackerPluginsConfiguration);
    /**
     * Calls each Plugin's `initialize` callback function, if defined
     */
    initialize(contexts: Required<ContextsConfig>): void;
    /**
     * Calls each Plugin's `beforeTransport` callback function, if defined
     */
    beforeTransport(contexts: Required<ContextsConfig>): void;
}
/**
 * A TypeScript friendly Object.keys
 */
declare const getObjectKeys: <T extends object>(obj: T) => (keyof T)[];
/**
 * A TypeScript generic describing an array with at least one item of the given Type
 */
type NonEmptyArray<T> = [
    T,
    ...T[]
];
/**
 * A TypeScript NonEmptyArray guard
 */
declare function isNonEmptyArray<T>(array: T[]): array is NonEmptyArray<T>;
/**
 * A UUID v4 generator
 */
declare const generateUUID: () => string;
/**
 * Executes the given predicate every `intervalMs` for a maximum of `timeoutMs`.
 * It resolves to `true` if predicated returns `true`. Resolves to false if `timeoutMs` is reached.
 */
declare const waitForPromise: ({ predicate, intervalMs, timeoutMs }: {
    predicate: Function;
    intervalMs: number;
    timeoutMs: number;
}) => Promise<boolean>;
/**
 * The TrackerQueueStoreConfig object.
 */
type TrackerQueueStoreConfig = {
    /**
     * Optional. A TrackerConsole instance for logging.
     */
    console?: TrackerConsole;
};
/**
 * Our Tracker Queue Store generic interface.
 */
interface TrackerQueueStoreInterface {
    /**
     * Optional. A TrackerConsole instance for logging.
     */
    readonly console?: TrackerConsole;
    /**
     * A name describing the Queue Store implementation for debugging purposes
     */
    readonly queueStoreName: string;
    /**
     * How many TrackerEvents are in the store
     */
    length: number;
    /**
     * Read Events from the store, if `size` is omitted all TrackerEvents will be returned
     */
    read(size?: number, filterPredicate?: (event: TrackerEvent) => boolean): Promise<TrackerEvent[]>;
    /**
     * Write Events to the store
     */
    write(...args: NonEmptyArray<TrackerEvent>): Promise<any>;
    /**
     * Delete TrackerEvents from the store by id
     */
    delete(TrackerEventIds: string[]): Promise<any>;
    /**
     * Delete all TrackerEvents from the store
     */
    clear(): Promise<any>;
}
/**
 * The definition of the runner function. Gets executed every batchDelayMs to process the Queue.
 */
type TrackerQueueProcessFunction = (...args: NonEmptyArray<TrackerEvent>) => Promise<any>;
/**
 * Our Tracker Events Queue generic interface.
 */
interface TrackerQueueInterface {
    /**
     * Optional. A TrackerConsole instance for logging.
     */
    readonly console?: TrackerConsole;
    /**
     * The TrackerQueueStore to use. Defaults to TrackerQueueMemoryStore
     */
    readonly store: TrackerQueueStoreInterface;
    /**
     * How many events to dequeue at the same time. Defaults to 10;
     */
    readonly batchSize: number;
    /**
     * How often to re-run and dequeue again, in ms. Defaults to 1000.
     */
    readonly batchDelayMs: number;
    /**
     * How many batches to process simultaneously. Defaults to 4.
     */
    readonly concurrency: number;
    /**
     * The function to execute every batchDelayMs. Must be set with `setProcessFunction` before calling `run`
     */
    processFunction?: TrackerQueueProcessFunction;
    /**
     * A name describing the Queue implementation for debugging purposes
     */
    readonly queueName: string;
    /**
     * Sets the processFunction to execute whenever run is called
     */
    setProcessFunction(processFunction: TrackerQueueProcessFunction): void;
    /**
     * Starts the runner process
     */
    startRunner(): void;
    /**
     * Adds one or more TrackerEvents to the Queue
     */
    push(...args: NonEmptyArray<TrackerEvent>): Promise<any>;
    /**
     * Adds one or more TrackerEvents to the Queue
     */
    readBatch(): Promise<TrackerEvent[]>;
    /**
     * Fetches a batch of Events from the Queue and executes the given `processFunction` with them.
     */
    run(): Promise<any>;
    /**
     * Empties the Queue
     */
    flush(): Promise<any>;
    /**
     * Returns whether all Events have been processed.
     */
    isIdle(): boolean;
}
/**
 * TrackerTransports can receive either Events ready to be processed or Event Promises.
 */
type TransportableEvent = TrackerEvent | Promise<TrackerEvent>;
/**
 * A custom error thrown by Sending Transports (eg: Fetch) whenever the Collector was not reachable.
 * RetryTransportAttempts will react to it by retrying.
 */
declare class TransportSendError extends ExtendableError {
}
/**
 * The configuration of TrackerTransportTransportSwitch
 */
type TrackerTransportConfig = {
    /**
     * Optional. A TrackerConsole instance for logging.
     */
    console?: TrackerConsole;
};
/**
 * The TrackerTransport interface provides a single function to handle one or more TrackerEvents.
 *
 * TrackerTransport implementations may vary depending on platform. Eg: web: fetch, node: https module, etc
 *
 * Also, simpler implementations can synchronously send TrackerEvents right away to the Collector while more complex
 * ones may leverage Queues, Workers and Storage for asynchronous sending, or batching.
 */
interface TrackerTransportInterface {
    /**
     * Optional. A TrackerConsole instance for logging.
     */
    readonly console?: TrackerConsole;
    /**
     * A name describing the Transport implementation for debugging purposes
     */
    readonly transportName: string;
    /**
     * Should return if the TrackerTransport can be used. Most useful in combination with TransportSwitch.
     */
    isUsable(): boolean;
    /**
     * Process one or more TransportableEvent. Eg. Send, queue, store, etc
     */
    handle(...args: NonEmptyArray<TransportableEvent>): Promise<any>;
}
/**
 * The configuration of the Tracker
 */
type TrackerConfig = ContextsConfig & {
    /**
     * Application ID. Used to generate ApplicationContext (global context).
     */
    applicationId: string;
    /**
     * Optional. Unique identifier for the TrackerInstance. Defaults to the same value of `applicationId`.
     */
    trackerId?: string;
    /**
     * Optional. TrackerQueue instance. Responsible for queueing and batching Events. Queuing occurs before Transport.
     */
    queue?: TrackerQueueInterface;
    /**
     * Optional. TrackerTransport instance. Responsible for sending or storing Events. Transport occurs after Queueing.
     */
    transport?: TrackerTransportInterface;
    /**
     * Optional. Plugins that will be executed when the Tracker initializes and before the Event is sent.
     */
    plugins?: TrackerPlugins;
    /**
     * Optional. A TrackerConsole instance for logging.
     */
    console?: TrackerConsole;
    /**
     * Optional. Determines if the TrackerInstance.trackEvent will process Events or not.
     */
    active?: boolean;
};
/**
 * The default list of Plugins of Web Tracker
 */
declare const makeTrackerDefaultPluginsList: (trackerConfig: TrackerConfig) => ApplicationContextPlugin[];
/**
 * The parameters object of Tracker.waitForQueue(parameters?: WaitForQueueParameters).
 */
type WaitForQueueParameters = {
    intervalMs?: number;
    timeoutMs?: number;
};
/**
 * The `options` parameter of Tracker.trackEvent(event: TrackerEventConfig, options?: TrackEventOptions).
 */
type TrackEventOptions = {
    waitForQueue?: true | WaitForQueueParameters;
    flushQueue?: true | "onTimeout";
};
/**
 * Our basic platform-agnostic JavaScript Tracker interface and implementation
 */
declare class Tracker implements Contexts, TrackerConfig {
    readonly console?: TrackerConsole;
    readonly applicationId: string;
    readonly trackerId: string;
    readonly queue?: TrackerQueueInterface;
    readonly transport?: TrackerTransportInterface;
    readonly plugins: TrackerPlugins;
    // By default, trackers are active
    active: boolean;
    // Contexts interface
    readonly location_stack: AbstractLocationContext[];
    readonly global_contexts: AbstractGlobalContext[];
    /**
     * Configures the Tracker instance via one TrackerConfig and optionally one or more ContextConfig.
     *
     * TrackerConfig is used to configure TrackerTransport and TrackerPlugins.
     *
     * ContextConfigs are used to configure location_stack and global_contexts. If multiple configurations have been
     * provided they will be merged onto each other to produce a single location_stack and global_contexts.
     */
    constructor(trackerConfig: TrackerConfig, ...contextConfigs: ContextsConfig[]);
    /**
     * Setter for the Tracker Instance `active` state, initializes Plugins and starts the Queue runner
     */
    setActive(newActiveState: boolean): void;
    /**
     * Flushes the Queue
     */
    flushQueue(): void;
    /**
     * Waits for Queue `isIdle` in an attempt to wait for it to finish its job.
     * Resolves regardless if the Queue reaches an idle state or timeout is reached.
     */
    waitForQueue(parameters?: WaitForQueueParameters): Promise<boolean>;
    /**
     * Merges Tracker Location and Global contexts, runs all Plugins and sends the Event via the TrackerTransport.
     */
    trackEvent(event: TrackerEventConfig, options?: TrackEventOptions): Promise<TrackerEvent>;
}
/**
 * The ApplicationContextPlugin Config object.
 */
type ApplicationContextPluginConfig = TrackerPluginConfig & Pick<TrackerConfig, "applicationId">;
/**
 * The ApplicationContext Plugin adds an ApplicationContext as GlobalContext before events are transported.
 */
declare class ApplicationContextPlugin implements TrackerPluginInterface {
    readonly console?: TrackerConsole;
    readonly pluginName = "ApplicationContextPlugin";
    readonly applicationContext: ApplicationContext;
    /**
     * Generates a ApplicationContext from the given config applicationId.
     */
    constructor(config: ApplicationContextPluginConfig);
    /**
     * Add the the ApplicationContext to the Event's Global Contexts
     */
    beforeTransport(contexts: Required<ContextsConfig>): void;
    /**
     * Make this plugin usable only if the Navigator API is available
     */
    isUsable(): boolean;
}
/**
 * Removes all properties starting with INTERNAL_PROPERTY_PREFIX from the given object and returns a new clean object.
 */
declare const cleanObjectFromInternalProperties: <T extends object>(obj: T) => {} & T;
/** Creates instance of ApplicationContext
 * @param {Object} props - factory properties
 * @param {string} props.id - A unique string identifier to be combined with the Context Type (`_type`)
 *         for Context instance uniqueness.
 * @returns {ApplicationContext} - ApplicationContext: A GlobalContext describing in which app the event happens, like a website or iOS app.
 */
declare const makeApplicationContext: (props: {
    id: string;
}) => ApplicationContext;
/** Creates instance of ContentContext
 * @param {Object} props - factory properties
 * @param {string} props.id - A unique string identifier to be combined with the Context Type (`_type`)
 *         for Context instance uniqueness.
 * @returns {ContentContext} - ContentContext: A Location Context that describes a logical section of the UI that contains other Location Contexts. Enabling Data Science to analyze this section specifically.
 */
declare const makeContentContext: (props: {
    id: string;
}) => ContentContext;
/** Creates instance of CookieIdContext
 * @param {Object} props - factory properties
 * @param {string} props.id - A unique string identifier to be combined with the Context Type (`_type`)
 *         for Context instance uniqueness.
 * @param {string} props.cookie_id - Unique identifier from the session cookie
 * @returns {CookieIdContext} - CookieIdContext: Global context with information needed to reconstruct a user session.
 */
declare const makeCookieIdContext: (props: {
    id: string;
    cookie_id: string;
}) => CookieIdContext;
/** Creates instance of ExpandableContext
 * @param {Object} props - factory properties
 * @param {string} props.id - A unique string identifier to be combined with the Context Type (`_type`)
 *         for Context instance uniqueness.
 * @returns {ExpandableContext} - ExpandableContext: A Location Context that describes a section of the UI that can expand & collapse.
 */
declare const makeExpandableContext: (props: {
    id: string;
}) => ExpandableContext;
/** Creates instance of HttpContext
 * @param {Object} props - factory properties
 * @param {string} props.id - A unique string identifier to be combined with the Context Type (`_type`)
 *         for Context instance uniqueness.
 * @param {string} props.referer - Full URL to HTTP referrer of the current page.
 * @param {string} props.user_agent - User-agent of the agent that sent the event.
 * @param {string} props.remote_address - (public) IP address of the agent that sent the event.
 * @returns {HttpContext} - HttpContext: A GlobalContext describing meta information about the agent that sent the event.
 */
declare const makeHttpContext: (props: {
    id: string;
    referer: string;
    user_agent: string;
    remote_address: string;
}) => HttpContext;
/** Creates instance of InputContext
 * @param {Object} props - factory properties
 * @param {string} props.id - A unique string identifier to be combined with the Context Type (`_type`)
 *         for Context instance uniqueness.
 * @returns {InputContext} - InputContext: A Location Context that describes an element that accepts user input, i.e. a form field.
 */
declare const makeInputContext: (props: {
    id: string;
}) => InputContext;
/** Creates instance of LinkContext
 * @param {Object} props - factory properties
 * @param {string} props.id - A unique string identifier to be combined with the Context Type (`_type`)
 *         for Context instance uniqueness.
 * @param {string} props.href - URL (href) the link points to.
 * @returns {LinkContext} - LinkContext: A PressableContext that contains an href.
 */
declare const makeLinkContext: (props: {
    id: string;
    href: string;
}) => LinkContext;
/** Creates instance of MediaPlayerContext
 * @param {Object} props - factory properties
 * @param {string} props.id - A unique string identifier to be combined with the Context Type (`_type`)
 *         for Context instance uniqueness.
 * @returns {MediaPlayerContext} - MediaPlayerContext: A Location Context that describes a section of the UI containing a media player.
 */
declare const makeMediaPlayerContext: (props: {
    id: string;
}) => MediaPlayerContext;
/** Creates instance of NavigationContext
 * @param {Object} props - factory properties
 * @param {string} props.id - A unique string identifier to be combined with the Context Type (`_type`)
 *         for Context instance uniqueness.
 * @returns {NavigationContext} - NavigationContext: A Location Context that describes a section of the UI containing navigational elements, for example a menu.
 */
declare const makeNavigationContext: (props: {
    id: string;
}) => NavigationContext;
/** Creates instance of OverlayContext
 * @param {Object} props - factory properties
 * @param {string} props.id - A unique string identifier to be combined with the Context Type (`_type`)
 *         for Context instance uniqueness.
 * @returns {OverlayContext} - OverlayContext: A Location Context that describes a section of the UI that represents an overlay, i.e. a Modal.
 * 	.
 */
declare const makeOverlayContext: (props: {
    id: string;
}) => OverlayContext;
/** Creates instance of PathContext
 * @param {Object} props - factory properties
 * @param {string} props.id - A unique string identifier to be combined with the Context Type (`_type`)
 *         for Context instance uniqueness.
 * @returns {PathContext} - PathContext: A GlobalContext describing the path where the user is when an event is sent.
 */
declare const makePathContext: (props: {
    id: string;
}) => PathContext;
/** Creates instance of PressableContext
 * @param {Object} props - factory properties
 * @param {string} props.id - A unique string identifier to be combined with the Context Type (`_type`)
 *         for Context instance uniqueness.
 * @returns {PressableContext} - PressableContext: An Location Context that describes an interactive element (like a link, button, icon),
 * 	that the user can press and will trigger an Interactive Event.
 */
declare const makePressableContext: (props: {
    id: string;
}) => PressableContext;
/** Creates instance of RootLocationContext
 * @param {Object} props - factory properties
 * @param {string} props.id - A unique string identifier to be combined with the Context Type (`_type`)
 *         for Context instance uniqueness.
 * @returns {RootLocationContext} - RootLocationContext: A Location Context that uniquely represents the top-level UI location of the user.
 */
declare const makeRootLocationContext: (props: {
    id: string;
}) => RootLocationContext;
/** Creates instance of SessionContext
 * @param {Object} props - factory properties
 * @param {string} props.id - A unique string identifier to be combined with the Context Type (`_type`)
 *         for Context instance uniqueness.
 * @param {number} props.hit_number - Hit counter relative to the current session, this event originated in.
 * @returns {SessionContext} - SessionContext: A GlobalContext describing meta information about the current session.
 */
declare const makeSessionContext: (props: {
    id: string;
    hit_number: number;
}) => SessionContext;
/** Creates instance of ApplicationLoadedEvent
 * @param {Object} props - factory properties
 * @param {AbstractLocationContext[]} props.location_stack - The location stack is an ordered list (stack), that contains a hierarchy of location contexts that
 *         deterministically describes where an event took place from global to specific.
 *         The whole stack (list) is needed to exactly pinpoint where in the UI the event originated.
 * @param {AbstractGlobalContext[]} props.global_contexts - Global contexts add global / general information about the event. They carry information that is not
 *         related to where the Event originated (location), such as device, platform or business data.
 * @returns {Omit<ApplicationLoadedEvent, 'id' | 'time'>} - ApplicationLoadedEvent: A NonInteractive event that is emitted after an application (eg. SPA) has finished loading.
 */
declare const makeApplicationLoadedEvent: (props?: {
    location_stack?: AbstractLocationContext[] | undefined;
    global_contexts?: AbstractGlobalContext[] | undefined;
} | undefined) => Omit<ApplicationLoadedEvent, "id" | "time">;
/** Creates instance of FailureEvent
 * @param {Object} props - factory properties
 * @param {AbstractLocationContext[]} props.location_stack - The location stack is an ordered list (stack), that contains a hierarchy of location contexts that
 *         deterministically describes where an event took place from global to specific.
 *         The whole stack (list) is needed to exactly pinpoint where in the UI the event originated.
 * @param {AbstractGlobalContext[]} props.global_contexts - Global contexts add global / general information about the event. They carry information that is not
 *         related to where the Event originated (location), such as device, platform or business data.
 * @param {string} props.message - Failure message.
 * @returns {Omit<FailureEvent, 'id' | 'time'>} - FailureEvent: A NonInteractiveEvent that is sent when a user action results in a error,
 * 	like an invalid email when sending a form.
 */
declare const makeFailureEvent: (props: {
    location_stack?: AbstractLocationContext[];
    global_contexts?: AbstractGlobalContext[];
    message: string;
}) => Omit<FailureEvent, "id" | "time">;
/** Creates instance of HiddenEvent
 * @param {Object} props - factory properties
 * @param {AbstractLocationContext[]} props.location_stack - The location stack is an ordered list (stack), that contains a hierarchy of location contexts that
 *         deterministically describes where an event took place from global to specific.
 *         The whole stack (list) is needed to exactly pinpoint where in the UI the event originated.
 * @param {AbstractGlobalContext[]} props.global_contexts - Global contexts add global / general information about the event. They carry information that is not
 *         related to where the Event originated (location), such as device, platform or business data.
 * @returns {Omit<HiddenEvent, 'id' | 'time'>} - HiddenEvent: A NonInteractiveEvent that's emitted after a LocationContext has become invisible.
 */
declare const makeHiddenEvent: (props?: {
    location_stack?: AbstractLocationContext[] | undefined;
    global_contexts?: AbstractGlobalContext[] | undefined;
} | undefined) => Omit<HiddenEvent, "id" | "time">;
/** Creates instance of InputChangeEvent
 * @param {Object} props - factory properties
 * @param {AbstractLocationContext[]} props.location_stack - The location stack is an ordered list (stack), that contains a hierarchy of location contexts that
 *         deterministically describes where an event took place from global to specific.
 *         The whole stack (list) is needed to exactly pinpoint where in the UI the event originated.
 * @param {AbstractGlobalContext[]} props.global_contexts - Global contexts add global / general information about the event. They carry information that is not
 *         related to where the Event originated (location), such as device, platform or business data.
 * @returns {Omit<InputChangeEvent, 'id' | 'time'>} - InputChangeEvent: Event triggered when user input is modified.
 */
declare const makeInputChangeEvent: (props?: {
    location_stack?: AbstractLocationContext[] | undefined;
    global_contexts?: AbstractGlobalContext[] | undefined;
} | undefined) => Omit<InputChangeEvent, "id" | "time">;
/** Creates instance of InteractiveEvent
 * @param {Object} props - factory properties
 * @param {AbstractLocationContext[]} props.location_stack - The location stack is an ordered list (stack), that contains a hierarchy of location contexts that
 *         deterministically describes where an event took place from global to specific.
 *         The whole stack (list) is needed to exactly pinpoint where in the UI the event originated.
 * @param {AbstractGlobalContext[]} props.global_contexts - Global contexts add global / general information about the event. They carry information that is not
 *         related to where the Event originated (location), such as device, platform or business data.
 * @returns {Omit<InteractiveEvent, 'id' | 'time'>} - InteractiveEvent: The parent of Events that are the direct result of a user interaction, e.g. a button click.
 */
declare const makeInteractiveEvent: (props?: {
    location_stack?: AbstractLocationContext[] | undefined;
    global_contexts?: AbstractGlobalContext[] | undefined;
} | undefined) => Omit<InteractiveEvent, "id" | "time">;
/** Creates instance of MediaEvent
 * @param {Object} props - factory properties
 * @param {AbstractLocationContext[]} props.location_stack - The location stack is an ordered list (stack), that contains a hierarchy of location contexts that
 *         deterministically describes where an event took place from global to specific.
 *         The whole stack (list) is needed to exactly pinpoint where in the UI the event originated.
 * @param {AbstractGlobalContext[]} props.global_contexts - Global contexts add global / general information about the event. They carry information that is not
 *         related to where the Event originated (location), such as device, platform or business data.
 * @returns {Omit<MediaEvent, 'id' | 'time'>} - MediaEvent: The parent of non-interactive events that are triggered by a media player.
 * 	It requires a MediaPlayerContext to detail the origin of the event.
 */
declare const makeMediaEvent: (props?: {
    location_stack?: AbstractLocationContext[] | undefined;
    global_contexts?: AbstractGlobalContext[] | undefined;
} | undefined) => Omit<MediaEvent, "id" | "time">;
/** Creates instance of MediaLoadEvent
 * @param {Object} props - factory properties
 * @param {AbstractLocationContext[]} props.location_stack - The location stack is an ordered list (stack), that contains a hierarchy of location contexts that
 *         deterministically describes where an event took place from global to specific.
 *         The whole stack (list) is needed to exactly pinpoint where in the UI the event originated.
 * @param {AbstractGlobalContext[]} props.global_contexts - Global contexts add global / general information about the event. They carry information that is not
 *         related to where the Event originated (location), such as device, platform or business data.
 * @returns {Omit<MediaLoadEvent, 'id' | 'time'>} - MediaLoadEvent: A MediaEvent that's emitted after a media item completes loading.
 */
declare const makeMediaLoadEvent: (props?: {
    location_stack?: AbstractLocationContext[] | undefined;
    global_contexts?: AbstractGlobalContext[] | undefined;
} | undefined) => Omit<MediaLoadEvent, "id" | "time">;
/** Creates instance of MediaPauseEvent
 * @param {Object} props - factory properties
 * @param {AbstractLocationContext[]} props.location_stack - The location stack is an ordered list (stack), that contains a hierarchy of location contexts that
 *         deterministically describes where an event took place from global to specific.
 *         The whole stack (list) is needed to exactly pinpoint where in the UI the event originated.
 * @param {AbstractGlobalContext[]} props.global_contexts - Global contexts add global / general information about the event. They carry information that is not
 *         related to where the Event originated (location), such as device, platform or business data.
 * @returns {Omit<MediaPauseEvent, 'id' | 'time'>} - MediaPauseEvent: A MediaEvent that's emitted after a media item pauses playback.
 */
declare const makeMediaPauseEvent: (props?: {
    location_stack?: AbstractLocationContext[] | undefined;
    global_contexts?: AbstractGlobalContext[] | undefined;
} | undefined) => Omit<MediaPauseEvent, "id" | "time">;
/** Creates instance of MediaStartEvent
 * @param {Object} props - factory properties
 * @param {AbstractLocationContext[]} props.location_stack - The location stack is an ordered list (stack), that contains a hierarchy of location contexts that
 *         deterministically describes where an event took place from global to specific.
 *         The whole stack (list) is needed to exactly pinpoint where in the UI the event originated.
 * @param {AbstractGlobalContext[]} props.global_contexts - Global contexts add global / general information about the event. They carry information that is not
 *         related to where the Event originated (location), such as device, platform or business data.
 * @returns {Omit<MediaStartEvent, 'id' | 'time'>} - MediaStartEvent: A MediaEvent that's emitted after a media item starts playback.
 */
declare const makeMediaStartEvent: (props?: {
    location_stack?: AbstractLocationContext[] | undefined;
    global_contexts?: AbstractGlobalContext[] | undefined;
} | undefined) => Omit<MediaStartEvent, "id" | "time">;
/** Creates instance of MediaStopEvent
 * @param {Object} props - factory properties
 * @param {AbstractLocationContext[]} props.location_stack - The location stack is an ordered list (stack), that contains a hierarchy of location contexts that
 *         deterministically describes where an event took place from global to specific.
 *         The whole stack (list) is needed to exactly pinpoint where in the UI the event originated.
 * @param {AbstractGlobalContext[]} props.global_contexts - Global contexts add global / general information about the event. They carry information that is not
 *         related to where the Event originated (location), such as device, platform or business data.
 * @returns {Omit<MediaStopEvent, 'id' | 'time'>} - MediaStopEvent: A MediaEvent that's emitted after a media item stops playback.
 */
declare const makeMediaStopEvent: (props?: {
    location_stack?: AbstractLocationContext[] | undefined;
    global_contexts?: AbstractGlobalContext[] | undefined;
} | undefined) => Omit<MediaStopEvent, "id" | "time">;
/** Creates instance of NonInteractiveEvent
 * @param {Object} props - factory properties
 * @param {AbstractLocationContext[]} props.location_stack - The location stack is an ordered list (stack), that contains a hierarchy of location contexts that
 *         deterministically describes where an event took place from global to specific.
 *         The whole stack (list) is needed to exactly pinpoint where in the UI the event originated.
 * @param {AbstractGlobalContext[]} props.global_contexts - Global contexts add global / general information about the event. They carry information that is not
 *         related to where the Event originated (location), such as device, platform or business data.
 * @returns {Omit<NonInteractiveEvent, 'id' | 'time'>} - NonInteractiveEvent: The parent of Events that are not directly triggered by a user action.
 */
declare const makeNonInteractiveEvent: (props?: {
    location_stack?: AbstractLocationContext[] | undefined;
    global_contexts?: AbstractGlobalContext[] | undefined;
} | undefined) => Omit<NonInteractiveEvent, "id" | "time">;
/** Creates instance of PressEvent
 * @param {Object} props - factory properties
 * @param {AbstractLocationContext[]} props.location_stack - The location stack is an ordered list (stack), that contains a hierarchy of location contexts that
 *         deterministically describes where an event took place from global to specific.
 *         The whole stack (list) is needed to exactly pinpoint where in the UI the event originated.
 * @param {AbstractGlobalContext[]} props.global_contexts - Global contexts add global / general information about the event. They carry information that is not
 *         related to where the Event originated (location), such as device, platform or business data.
 * @returns {Omit<PressEvent, 'id' | 'time'>} - PressEvent: An InteractiveEvent that is sent when a user presses on a pressable element
 * 	(like a link, button, icon).
 */
declare const makePressEvent: (props?: {
    location_stack?: AbstractLocationContext[] | undefined;
    global_contexts?: AbstractGlobalContext[] | undefined;
} | undefined) => Omit<PressEvent, "id" | "time">;
/** Creates instance of SuccessEvent
 * @param {Object} props - factory properties
 * @param {AbstractLocationContext[]} props.location_stack - The location stack is an ordered list (stack), that contains a hierarchy of location contexts that
 *         deterministically describes where an event took place from global to specific.
 *         The whole stack (list) is needed to exactly pinpoint where in the UI the event originated.
 * @param {AbstractGlobalContext[]} props.global_contexts - Global contexts add global / general information about the event. They carry information that is not
 *         related to where the Event originated (location), such as device, platform or business data.
 * @param {string} props.message - Success message.
 * @returns {Omit<SuccessEvent, 'id' | 'time'>} - SuccessEvent: A NonInteractiveEvent that is sent when a user action is successfully completed,
 * 	like sending an email form.
 */
declare const makeSuccessEvent: (props: {
    location_stack?: AbstractLocationContext[];
    global_contexts?: AbstractGlobalContext[];
    message: string;
}) => Omit<SuccessEvent, "id" | "time">;
/** Creates instance of VisibleEvent
 * @param {Object} props - factory properties
 * @param {AbstractLocationContext[]} props.location_stack - The location stack is an ordered list (stack), that contains a hierarchy of location contexts that
 *         deterministically describes where an event took place from global to specific.
 *         The whole stack (list) is needed to exactly pinpoint where in the UI the event originated.
 * @param {AbstractGlobalContext[]} props.global_contexts - Global contexts add global / general information about the event. They carry information that is not
 *         related to where the Event originated (location), such as device, platform or business data.
 * @returns {Omit<VisibleEvent, 'id' | 'time'>} - VisibleEvent: A NonInteractiveEvent that's emitted after a section LocationContext has become visible.
 */
declare const makeVisibleEvent: (props?: {
    location_stack?: AbstractLocationContext[] | undefined;
    global_contexts?: AbstractGlobalContext[] | undefined;
} | undefined) => Omit<VisibleEvent, "id" | "time">;
/**
 * Converts a Location Stack onto its Location Path
 */
declare const getLocationPath: (locationStack: LocationStack) => string;
/**
 * Represents a Location collision. Returned by TrackerState.addElementLocation whenever the uniqueness check fails.
 */
type LocationCollision = {
    locationPath: string;
    existingElementId: string;
    collidingElementId: string;
};
/**
 * Provides a global state to store and retrieve Element's Location Stacks, as well as validating their uniqueness
 */
declare const TrackerElementLocations: {
    elementLocations: Map<string, string[]>;
    add: ({ elementId, locationPath }: {
        elementId: string;
        locationPath: string;
    }) => true | LocationCollision;
    delete: (elementId: string | undefined | null) => boolean;
    clear: () => void;
};
/**
 * The configuration of a TrackerQueue
 */
type TrackerQueueConfig = {
    /**
     * Optional. A TrackerConsole instance for logging.
     */
    console?: TrackerConsole;
    /**
     * Optional. The TrackerQueueStore to use. Defaults to TrackerQueueMemoryStore
     */
    store?: TrackerQueueStoreInterface;
    /**
     * Optional. How many events to dequeue at the same time. Defaults to 10;
     */
    batchSize?: number;
    /**
     * Optional. How often to re-run and dequeue again, in ms. Defaults to 1000.
     */
    batchDelayMs?: number;
    /**
     * Optional. How many batches to process simultaneously. Defaults to 4.
     */
    concurrency?: number;
};
/**
 * A very simple Batched Queue implementation.
 */
declare class TrackerQueue implements TrackerQueueInterface {
    readonly console?: TrackerConsole;
    readonly queueName = "TrackerQueue";
    processFunction?: TrackerQueueProcessFunction;
    readonly store: TrackerQueueStoreInterface;
    readonly batchSize: number;
    readonly batchDelayMs: number;
    readonly concurrency: number;
    // Hold a list of Event IDs that are currently being processed
    processingEventIds: string[];
    /**
     * Initializes batching configuration with some sensible values.
     */
    constructor(config?: TrackerQueueConfig);
    setProcessFunction(processFunction: TrackerQueueProcessFunction): void;
    startRunner(): void;
    push(...args: NonEmptyArray<TrackerEvent>): Promise<any>;
    readBatch(): Promise<TrackerEvent[]>;
    run(): Promise<any>;
    flush(): Promise<any>;
    isIdle(): boolean;
}
/**
 * An in-memory implementation of a TrackerQueueStore.
 */
declare class TrackerQueueMemoryStore implements TrackerQueueStoreInterface {
    readonly console?: TrackerConsole;
    queueStoreName: string;
    events: TrackerEvent[];
    constructor(config?: TrackerQueueStoreConfig);
    read(size?: number, filterPredicate?: (event: TrackerEvent) => boolean): Promise<TrackerEvent[]>;
    write(...args: NonEmptyArray<TrackerEvent>): Promise<any>;
    delete(trackerEventIds: string[]): Promise<any>;
    clear(): Promise<any>;
    get length(): number;
}
/**
 * The generic interface of the TrackerRepository
 */
interface TrackerRepositoryInterface<T extends Tracker> {
    /**
     * A Map of all Tracker instances by `trackerId`
     */
    trackersMap: Map<string, T>;
    /**
     * The default Tracker instance that will be returned when invoking `getTracker` without `trackerId` parameter
     */
    defaultTracker?: T;
    /**
     * Adds a new Tracker instance to the trackersMap
     */
    add(newInstance: T): void;
    /**
     * Deletes Tracker instance from the trackersMap by `trackerId`
     */
    delete(trackerId: string): void;
    /**
     * Get a Tracker instance from the trackersMap by `trackerId`
     */
    get(trackerId?: string): T | undefined;
    /**
     * Changes the default Tracker instance by specifying a `trackerId`
     */
    setDefault(trackerId: string): void;
    /**
     * Sets all Tracker instances as active
     */
    activateAll(): void;
    /**
     * Sets all Tracker instances as inactive
     */
    deactivateAll(): void;
    /**
     * Flushes all Tracker instances Queues
     */
    flushAllQueues(): void;
}
/**
 * TrackerRepository allows developers to create and use multiple Tracker instances in the same Application.
 */
declare class TrackerRepository<T extends Tracker> implements TrackerRepositoryInterface<T> {
    trackersMap: Map<string, T>;
    defaultTracker?: T;
    add(newInstance: T): void;
    delete(trackerId: string): void;
    get(trackerId?: string): T | undefined;
    setDefault(trackerId: string): void;
    activateAll(): void;
    deactivateAll(): void;
    flushAllQueues(): void;
}
/**
 * The configuration of TrackerTransportGroup.
 */
type TransportGroupConfig = TrackerTransportConfig & {
    transports: NonEmptyArray<TrackerTransportInterface>;
};
/**
 * TrackerTransportGroup provides a mechanism to hand over TrackerEvents to multiple transports. The group is usable
 * if at least one of the given TrackerTransports is usable.
 *
 * This can be used when having multiple Collectors but also for simpler development needs, such as handling & logging
 */
declare class TrackerTransportGroup implements TrackerTransportInterface {
    readonly console?: TrackerConsole;
    readonly transportName = "TrackerTransportGroup";
    readonly usableTransports: TrackerTransportInterface[];
    /**
     * Filter and store the list of usable transports, received as construction parameters, in state
     */
    constructor(config: TransportGroupConfig);
    /**
     * Simply proxy the `handle` method to all the usable TrackerTransport instances we have.
     */
    handle(...args: NonEmptyArray<TransportableEvent>): Promise<any>;
    /**
     * The whole TransportGroup is usable if we found at least one usable TrackerTransport
     */
    isUsable(): boolean;
}
/**
 * The Interface of RetryTransportAttempts
 */
type TrackerTransportRetryAttemptInterface = TrackerTransportConfig & Required<TrackerTransportRetryConfig>;
/**
 * A RetryTransportAttempt is a TransportRetry worker.
 * TransportRetry creates a RetryTransportAttempt instance whenever its `handle` method is invoked.
 */
declare class TrackerTransportRetryAttempt implements TrackerTransportRetryAttemptInterface {
    readonly console?: TrackerConsole;
    // RetryTransport State
    readonly transport: TrackerTransportInterface;
    readonly maxAttempts: number;
    readonly maxRetryMs: number;
    readonly minTimeoutMs: number;
    readonly maxTimeoutMs: number;
    readonly retryFactor: number;
    /**
     * The list of Events to handle.
     */
    events: NonEmptyArray<TransportableEvent>;
    /**
     * A list of errors in reverse order. Eg: element 0 is the last error occurred. N-1 is the first.
     */
    errors: Error[];
    /**
     * How many times we have tried so far. Used in the calculation of the exponential backoff.
     */
    attemptCount: number;
    /**
     * Start time is persisted before each attempt and checked before retrying to verify if we exceeded maxRetryMs.
     */
    startTime: number;
    constructor(retryTransportInstance: TrackerTransportRetry, events: NonEmptyArray<TransportableEvent>);
    /**
     * Determines how much time we have to wait based on the number of attempts and the configuration variables
     */
    calculateNextTimeoutMs(attemptCount: number): number;
    /**
     * Verifies if we exceeded maxAttempts or maxRetryMs before attempting to call the given Transport's `handle` method.
     * If promise rejections occur it invokes the `retry` method.
     */
    run(): Promise<any>;
    /**
     * Handles the given error, waits for the appropriate time and `attempt`s again
     */
    retry(error: Error): Promise<any>;
}
/**
 * The configuration object of a RetryTransport. Requires a TrackerTransport instance.
 */
type TrackerTransportRetryConfig = {
    /**
     * The TrackerTransport instance to wrap around. Retry Transport will invoke the wrapped TrackerTransport `handle`.
     */
    transport: TrackerTransportInterface;
    /**
     * Optional. How many times we will retry before giving up. Defaults to 10;
     */
    maxAttempts?: number;
    /**
     * Optional. How much time we will keep retrying before giving up. Defaults to Infinity;
     */
    maxRetryMs?: number;
    /**
     * Optional. The following properties are used to calculate the exponential timeouts between attempts.
     *
     * Given an `attemptCount`, representing how many times we retried so far, this is the formula:
     *   min( round( minTimeoutMs * pow(retryFactor, attemptCount) ), maxTimeoutMs)
     */
    minTimeoutMs?: number; // defaults to 1000
    maxTimeoutMs?: number; // defaults to Infinity
    retryFactor?: number; // defaults to 2
};
/**
 * A TrackerTransport implementing exponential backoff retries around a TrackerTransport handle method.
 * It allows to also specify maximum retry time and number of attempts.
 */
declare class TrackerTransportRetry implements TrackerTransportInterface {
    readonly console?: TrackerConsole;
    readonly transportName = "TrackerTransportRetry";
    // RetryTransportConfig
    readonly transport: TrackerTransportInterface;
    readonly maxAttempts: number;
    readonly maxRetryMs: number;
    readonly minTimeoutMs: number;
    readonly maxTimeoutMs: number;
    readonly retryFactor: number;
    // A list of attempts that are currently running
    attempts: TrackerTransportRetryAttempt[];
    constructor(config: TrackerTransportConfig & TrackerTransportRetryConfig);
    /**
     * Creates a RetryTransportAttempt for the given Events and runs it.
     */
    handle(...args: NonEmptyArray<TransportableEvent>): Promise<any>;
    isUsable(): boolean;
}
/**
 * The configuration of TrackerTransportSwitch.
 */
type TrackerTransportSwitchConfig = TrackerTransportConfig & {
    transports: NonEmptyArray<TrackerTransportInterface>;
};
/**
 * TrackerTransportSwitch provides a fallback mechanism to pick the first usable transport in a list of them.
 * The switch is usable if at least one of the given TrackerTransports is usable.
 *
 * This mechanism can be used to configure multiple TrackerTransport instances, in order of preference, and
 * have TransportSwitch test each of them via the `isUsable` method to determine the topmost usable one.
 */
declare class TrackerTransportSwitch implements TrackerTransportInterface {
    readonly console?: TrackerConsole;
    readonly transportName = "TrackerTransportSwitch";
    readonly firstUsableTransport?: TrackerTransportInterface;
    /**
     * Finds the first TrackerTransport which `isUsable()`.
     */
    constructor(config: TrackerTransportSwitchConfig);
    /**
     * Simply proxy the `handle` method to the usable TrackerTransport we found during construction, if any
     */
    handle(...args: NonEmptyArray<TransportableEvent>): Promise<any>;
    /**
     * The whole TransportSwitch is usable if we found a usable TrackerTransport
     */
    isUsable(): boolean;
}
/**
 * The PathContextFromURL Plugin gathers the current URL using the Location API.
 * It implements the `beforeTransport` lifecycle method. This ensures the URL is retrieved before each Event is sent.
 */
declare class PathContextFromURLPlugin implements TrackerPluginInterface {
    readonly console?: TrackerConsole;
    readonly pluginName = "PathContextFromURLPlugin";
    /**
     * The constructor is merely responsible for processing the given TrackerPluginConfiguration `console` parameter.
     */
    constructor(config?: TrackerPluginConfig);
    /**
     * Generate a fresh PathContext before each TrackerEvent is handed over to the TrackerTransport.
     */
    beforeTransport(contexts: Required<ContextsConfig>): void;
    /**
     * Make this plugin usable only on web, eg: Document and Location APIs are both available
     */
    isUsable(): boolean;
}
/**
 * Browser Tracker can be configured in a easier way, as opposed to the core tracker, by specifying just an `endpoint`.
 * Internally it will automatically configure the Transport layer for the given `endpoint` with sensible defaults.
 * It also accepts a number of options to configure automatic tracking behavior:
 */
type BrowserTrackerConfig = TrackerConfig & {
    /**
     * The collector endpoint URL.
     */
    endpoint?: string;
    /**
     * Optional. Whether to track application loaded events automatically. Enabled by default.
     */
    trackApplicationLoadedEvent?: boolean;
};
/**
 * The default list of Plugins of Browser Tracker
 */
declare const makeDefaultPluginsList: (trackerConfig: BrowserTrackerConfig) => (ApplicationContextPlugin | PathContextFromURLPlugin)[];
/**
 * A factory to create the default Queue of Browser Tracker.
 */
declare const makeDefaultQueue: (trackerConfig: BrowserTrackerConfig) => TrackerQueueInterface;
/**
 * A factory to create the default Transport of Browser Tracker.
 */
declare const makeDefaultTransport: (trackerConfig: BrowserTrackerConfig) => TrackerTransportInterface;
/**
 * The type of Elements the type guards can work with
 */
type GuardableElement = Node | EventTarget | null;
/**
 * A Taggable Element is an HTMLElement or an SVGElement
 */
type TaggableElement = HTMLElement | SVGElement;
/**
 * All the attributes that are added to a DOM Element to make it trackable
 */
declare enum TaggingAttribute {
    // A unique identifier used internally to pinpoint to a specific instance of a tagged element
    elementId = "data-objectiv-element-id",
    // DOM traversing to rebuild Locations is not always possible, eg: Portals. This allows specifying a parent Element.
    parentElementId = "data-objectiv-parent-element-id",
    // A serialized instance of an Objectiv Context.
    context = "data-objectiv-context",
    // Track click events for this tagged element.
    trackClicks = "data-objectiv-track-clicks",
    // Track blur events for this tagged element.
    trackBlurs = "data-objectiv-track-blurs",
    // Determines how we will track visibility events for this tagged element.
    trackVisibility = "data-objectiv-track-visibility",
    // A list of serialized ChildrenTaggingQuery objects.
    tagChildren = "data-objectiv-tag-children",
    // Set to `true` by the Mutation Observer when an Element has been processed for auto-tracking.
    tracked = "data-objectiv-tracked",
    // Holds validation options. They may be used to disable or configure certain automatic checks.
    validate = "data-objectiv-validate"
}
/**
 * The object that Location Taggers return, stringified
 */
declare const TagLocationAttributes: import("superstruct").Struct<{
    "data-objectiv-element-id": string;
    "data-objectiv-context": string;
    "data-objectiv-parent-element-id"?: string | undefined;
    "data-objectiv-track-clicks"?: string | undefined;
    "data-objectiv-track-blurs"?: string | undefined;
    "data-objectiv-track-visibility"?: string | undefined;
    "data-objectiv-validate"?: string | undefined;
}, {
    "data-objectiv-element-id": import("superstruct").Struct<string, null>;
    "data-objectiv-parent-element-id": import("superstruct").Struct<string | undefined, null>;
    "data-objectiv-context": import("superstruct").Struct<string, null>;
    "data-objectiv-track-clicks": import("superstruct").Struct<string | undefined, null>;
    "data-objectiv-track-blurs": import("superstruct").Struct<string | undefined, null>;
    "data-objectiv-track-visibility": import("superstruct").Struct<string | undefined, null>;
    "data-objectiv-validate": import("superstruct").Struct<string | undefined, null>;
}>;
type TagLocationAttributes = Infer<typeof TagLocationAttributes>;
/**
 * A ParentTaggedElement is a TaggedElement with the TaggingAttribute.parentElementId
 */
type ParentTaggedElement = TaggableElement & Pick<TagLocationAttributes, TaggingAttribute.parentElementId>;
/**
 * A type guard to determine if the given Element is a TaggableElement decorated with LocationTaggingAttributes.parentElementId.
 */
declare const isParentTaggedElement: (element: GuardableElement) => element is ParentTaggedElement;
/**
 * The stringified version of the `ChildrenTaggingAttributes`
 */
declare const TagChildrenAttributes: import("superstruct").Struct<{
    "data-objectiv-tag-children": string;
}, {
    "data-objectiv-tag-children": import("superstruct").Struct<string, null>;
}>;
type TagChildrenAttributes = Infer<typeof TagChildrenAttributes>;
/**
 * A TagChildrenElement is a TaggableElement already decorated with our ChildrenTaggingAttributes
 */
type TagChildrenElement = TaggableElement & TagChildrenAttributes;
/**
 * A type guard to determine if the given Element is a TaggableElement decorated with ChildrenLocationTaggingAttributes.
 */
declare const isTagChildrenElement: (element: GuardableElement) => element is TagChildrenElement;
/**
 * A type guard to determine if a the given Element is an HTMLElement or SVGElement.
 * In general we can only tag Elements supporting dataset attributes.
 */
declare const isTaggableElement: (element: GuardableElement) => element is TaggableElement;
/**
 * A TaggedElement is a TaggableElement already decorated with our LocationTaggingAttributes
 */
type TaggedElement = TaggableElement & TagLocationAttributes;
/**
 * A type guard to determine if the given Element is a TaggableElement decorated with LocationTaggingAttributes.
 * Note: For performance and simplicity we only check if `context` is present. Assume all other attributes are there.
 */
declare const isTaggedElement: (element: GuardableElement) => element is TaggedElement;
/**
 * ChildrenTaggingAttribute parser
 */
declare const parseTagChildren: (stringifiedChildrenTaggingAttribute: string | null) => {
    queryAll: string;
    tagAs: {
        "data-objectiv-element-id": string;
        "data-objectiv-context": string;
        "data-objectiv-parent-element-id"?: string | undefined;
        "data-objectiv-track-clicks"?: string | undefined;
        "data-objectiv-track-blurs"?: string | undefined;
        "data-objectiv-track-visibility"?: string | undefined;
        "data-objectiv-validate"?: string | undefined;
    };
}[];
/**
 * JSON Objects parser
 */
declare const parseJson: <T = unknown>(stringifiedObject: string | null, struct: Struct<T, unknown>) => T;
/**
 * LocationContexts parser
 */
declare const parseLocationContext: (stringifiedContext: string | null) => {
    _type: "RootLocationContext";
    id: string;
    __location_context: true;
} | {
    _type: "ContentContext";
    id: string;
    __location_context: true;
} | {
    _type: "ExpandableContext";
    id: string;
    __location_context: true;
} | {
    _type: "MediaPlayerContext";
    id: string;
    __location_context: true;
} | {
    _type: "NavigationContext";
    id: string;
    __location_context: true;
} | {
    _type: "OverlayContext";
    id: string;
    __location_context: true;
} | {
    _type: "InputContext";
    id: string;
    __location_context: true;
} | {
    _type: "PressableContext";
    id: string;
    __location_context: true;
    __pressable_context: true;
} | {
    _type: "LinkContext";
    id: string;
    __location_context: true;
    __pressable_context: true;
    href: string;
};
/**
 * The Options attribute of the TrackClicks TaggingAttribute
 */
declare const TrackClicksOptions: import("superstruct").Struct<{
    waitForQueue?: {
        intervalMs?: number | undefined;
        timeoutMs?: number | undefined;
    } | undefined;
    flushQueue?: boolean | "onTimeout" | undefined;
} | undefined, null>;
type TrackClicksOptions = Infer<typeof TrackClicksOptions>;
/**
 * `trackClicks` Tagging Attribute to TrackClicksOptions parser
 * Differently than other simplistic parsers, this one transforms the `trackClicks` attribute in a different format.
 */
declare const parseTrackClicks: (stringifiedTrackClicksAttribute: string | null) => TrackClicksOptions;
/**
 * `trackVisibility` Tagging Attribute parser
 */
declare const parseTrackVisibility: (stringifiedTrackVisibilityAttribute: string | null) => {
    mode: "auto";
} | {
    mode: "manual";
    isVisible: boolean;
};
/**
 * `validate` Tagging Attribute parser
 */
declare const parseValidate: (stringifiedValidateAttribute: string | null) => {
    locationUniqueness: boolean;
};
/**
 * ChildrenTaggingAttribute stringifier
 */
declare const stringifyTagChildren: (queries: {
    queryAll: string;
    tagAs?: {
        "data-objectiv-element-id": string;
        "data-objectiv-context": string;
        "data-objectiv-parent-element-id"?: string | undefined;
        "data-objectiv-track-clicks"?: string | undefined;
        "data-objectiv-track-blurs"?: string | undefined;
        "data-objectiv-track-visibility"?: string | undefined;
        "data-objectiv-validate"?: string | undefined;
    } | undefined;
}[]) => string;
/**
 * JSON Objects stringifier
 */
declare const stringifyJson: <T = unknown>(object: T, struct: Struct<T, unknown>) => string;
/**
 * Abstract Context struct
 */
declare const AbstractContext: import("superstruct").Struct<{
    _type: string;
    id: string;
}, {
    id: import("superstruct").Struct<string, null>;
    _type: import("superstruct").Struct<string, null>;
}>;
/**
 * AbstractLocationContext struct
 */
declare const AbstractLocationContext$0: import("superstruct").Struct<{
    _type: string;
    id: string;
    __location_context: true;
}, {
    __location_context: import("superstruct").Struct<true, true>;
    _type: import("superstruct").Struct<string, null>;
    id: import("superstruct").Struct<string, null>;
}>;
/**
 * AbstractPressableContext struct
 */
declare const AbstractPressableContext: import("superstruct").Struct<{
    _type: string;
    id: string;
    __location_context: true;
    __pressable_context: true;
}, {
    __pressable_context: import("superstruct").Struct<true, true>;
    _type: import("superstruct").Struct<string, null>;
    id: import("superstruct").Struct<string, null>;
    __location_context: import("superstruct").Struct<true, true>;
}>;
/**
 * RootLocationContext struct
 */
declare const RootLocationContext$0: import("superstruct").Struct<{
    _type: "RootLocationContext";
    id: string;
    __location_context: true;
}, {
    _type: import("superstruct").Struct<"RootLocationContext", "RootLocationContext">;
    id: import("superstruct").Struct<string, null>;
    __location_context: import("superstruct").Struct<true, true>;
}>;
/**
 * ContentContext struct
 */
declare const ContentContext$0: import("superstruct").Struct<{
    _type: "ContentContext";
    id: string;
    __location_context: true;
}, {
    _type: import("superstruct").Struct<"ContentContext", "ContentContext">;
    id: import("superstruct").Struct<string, null>;
    __location_context: import("superstruct").Struct<true, true>;
}>;
/**
 * ExpandableContext struct
 */
declare const ExpandableContext$0: import("superstruct").Struct<{
    _type: "ExpandableContext";
    id: string;
    __location_context: true;
}, {
    _type: import("superstruct").Struct<"ExpandableContext", "ExpandableContext">;
    id: import("superstruct").Struct<string, null>;
    __location_context: import("superstruct").Struct<true, true>;
}>;
/**
 * MediaPlayerContext struct
 */
declare const MediaPlayerContext$0: import("superstruct").Struct<{
    _type: "MediaPlayerContext";
    id: string;
    __location_context: true;
}, {
    _type: import("superstruct").Struct<"MediaPlayerContext", "MediaPlayerContext">;
    id: import("superstruct").Struct<string, null>;
    __location_context: import("superstruct").Struct<true, true>;
}>;
/**
 * NavigationContext struct
 */
declare const NavigationContext$0: import("superstruct").Struct<{
    _type: "NavigationContext";
    id: string;
    __location_context: true;
}, {
    _type: import("superstruct").Struct<"NavigationContext", "NavigationContext">;
    id: import("superstruct").Struct<string, null>;
    __location_context: import("superstruct").Struct<true, true>;
}>;
/**
 * OverlayContext struct
 */
declare const OverlayContext$0: import("superstruct").Struct<{
    _type: "OverlayContext";
    id: string;
    __location_context: true;
}, {
    _type: import("superstruct").Struct<"OverlayContext", "OverlayContext">;
    id: import("superstruct").Struct<string, null>;
    __location_context: import("superstruct").Struct<true, true>;
}>;
/**
 * InputContext struct
 */
declare const InputContext$0: import("superstruct").Struct<{
    _type: "InputContext";
    id: string;
    __location_context: true;
}, {
    _type: import("superstruct").Struct<"InputContext", "InputContext">;
    id: import("superstruct").Struct<string, null>;
    __location_context: import("superstruct").Struct<true, true>;
}>;
/**
 * OverlayContext struct
 */
declare const PressableContext$0: import("superstruct").Struct<{
    _type: "PressableContext";
    id: string;
    __location_context: true;
    __pressable_context: true;
}, {
    _type: import("superstruct").Struct<"PressableContext", "PressableContext">;
    id: import("superstruct").Struct<string, null>;
    __location_context: import("superstruct").Struct<true, true>;
    __pressable_context: import("superstruct").Struct<true, true>;
}>;
/**
 * LinkContext struct
 */
declare const LinkContext$0: import("superstruct").Struct<{
    _type: "LinkContext";
    id: string;
    __location_context: true;
    __pressable_context: true;
    href: string;
}, {
    _type: import("superstruct").Struct<"LinkContext", "LinkContext">;
    href: import("superstruct").Struct<string, null>;
    id: import("superstruct").Struct<string, null>;
    __location_context: import("superstruct").Struct<true, true>;
    __pressable_context: import("superstruct").Struct<true, true>;
}>;
/**
 * Struct union to match any LocationContext
 */
declare const AnyLocationContext: import("superstruct").Struct<{
    _type: "RootLocationContext";
    id: string;
    __location_context: true;
} | {
    _type: "ContentContext";
    id: string;
    __location_context: true;
} | {
    _type: "ExpandableContext";
    id: string;
    __location_context: true;
} | {
    _type: "MediaPlayerContext";
    id: string;
    __location_context: true;
} | {
    _type: "NavigationContext";
    id: string;
    __location_context: true;
} | {
    _type: "OverlayContext";
    id: string;
    __location_context: true;
} | {
    _type: "InputContext";
    id: string;
    __location_context: true;
} | {
    _type: "PressableContext";
    id: string;
    __location_context: true;
    __pressable_context: true;
} | {
    _type: "LinkContext";
    id: string;
    __location_context: true;
    __pressable_context: true;
    href: string;
}, null>;
type AnyLocationContext = Infer<typeof AnyLocationContext>;
/**
 * Struct union to match any PressableContext
 */
declare const AnyPressableContext: import("superstruct").Struct<{
    _type: "PressableContext";
    id: string;
    __location_context: true;
    __pressable_context: true;
} | {
    _type: "LinkContext";
    id: string;
    __location_context: true;
    __pressable_context: true;
    href: string;
}, null>;
type AnyPressableContext = Infer<typeof AnyPressableContext>;
/**
 * Struct union to match any Clickable Context, that is AnyPressableContext + ExpandableContext
 */
declare const AnyClickableContext: import("superstruct").Struct<{
    _type: "ExpandableContext";
    id: string;
    __location_context: true;
} | {
    _type: "PressableContext";
    id: string;
    __location_context: true;
    __pressable_context: true;
} | {
    _type: "LinkContext";
    id: string;
    __location_context: true;
    __pressable_context: true;
    href: string;
}, null>;
type AnyClickableContext = Infer<typeof AnyClickableContext>;
/**
 * Struct union to match any Showable Context, that is Overlays and ExpandableContext
 */
declare const AnyShowableContext: import("superstruct").Struct<{
    _type: "ExpandableContext";
    id: string;
    __location_context: true;
} | {
    _type: "OverlayContext";
    id: string;
    __location_context: true;
}, null>;
type AnyShowableContext = Infer<typeof AnyShowableContext>;
/**
 * LocationContexts stringifier
 */
declare const stringifyLocationContext: (contextObject: AnyLocationContext) => string;
/**
 * The definition for the `trackClicks` Tagging Attribute
 */
declare const TrackClicksAttribute: import("superstruct").Struct<boolean | {
    waitUntilTracked: true | {
        intervalMs?: number | undefined;
        timeoutMs?: number | undefined;
        flushQueue?: boolean | "onTimeout" | undefined;
    };
}, null>;
type TrackClicksAttribute = Infer<typeof TrackClicksAttribute>;
/**
 * `trackClicks` Tagging Attribute stringifier
 */
declare const stringifyTrackClicks: (trackClicksAttribute: TrackClicksAttribute) => string;
/**
 * The definition of the `trackVisibility` Tagging Attribute
 */
declare const TrackVisibilityAttribute: import("superstruct").Struct<{
    mode: "auto";
} | {
    mode: "manual";
    isVisible: boolean;
}, null>;
type TrackVisibilityAttribute = Infer<typeof TrackVisibilityAttribute>;
/**
 * `trackVisibility` Tagging Attribute stringifier
 */
declare const stringifyTrackVisibility: (trackVisibilityAttribute: TrackVisibilityAttribute) => string;
/**
 * The definition of the `validate` Tagging Attribute
 */
declare const ValidateAttribute: import("superstruct").Struct<{
    locationUniqueness: boolean;
}, {
    locationUniqueness: import("superstruct").Struct<boolean, null>;
}>;
type ValidateAttribute = Infer<typeof ValidateAttribute>;
/**
 * `validate` Tagging Attribute stringifier
 */
declare const stringifyValidate: (validateAttribute: ValidateAttribute) => string;
/**
 * Compares two TrackerConfig objects ignoring mutable attributes.
 * Returns true if configurations are equivalent, false otherwise.
 */
declare const compareTrackerConfigs: (trackerConfigA: TrackerConfig, trackerConfigB: TrackerConfig) => boolean;
/**
 * Walk the DOM upwards looking for Tagged Elements. The resulting array can be used to reconstruct a Location Stack.
 */
declare const findParentTaggedElements: (element: TaggableElement | null, parentElements?: TaggedElement[]) => TaggedElement[];
/**
 * Browser Tracker is a Tracker Core constructor with simplified parameters and some preconfigured Plugins.
 * It initializes with a Queued Fetch and XMLHttpRequest Transport Switch wrapped in a Retry Transport automatically.
 * The resulting Queue has some sensible defaults (10 events every 100ms) for sending events in batches.
 * The Retry logic is configured for 10 retries with exponential backoff starting at 1000ms.
 *
 * This statement:
 *
 *  const tracker = new BrowserTracker({ applicationId: 'app-id', endpoint: '/endpoint', console: console });
 *
 * is equivalent to:
 *
 *  const trackerId = trackerConfig.trackerId ?? trackerConfig.applicationId;
 *  const console = trackerConfig.console;
 *  const fetchTransport = new FetchTransport({ endpoint: '/endpoint', console });
 *  const xhrTransport = new XHRTransport({ endpoint: '/endpoint', console });
 *  const transportSwitch = new TransportSwitch({ transports: [fetchTransport, xhrTransport], console });
 *  const transport = new RetryTransport({ transport: transportSwitch, console });
 *  const queueStorage = new LocalStorageQueueStore({ trackerId, console })
 *  const trackerQueue = new TrackerQueue({ storage: trackerStorage, console });
 *  const applicationContextPlugin = new ApplicationContextPlugin({ applicationId: 'app-id', console });
 *  const pathContextFromURLPlugin = new PathContextFromURLPlugin({ console });
 *  const plugins = new TrackerPlugins({ plugins: [ applicationContextPlugin, pathContextFromURLPlugin ], console });
 *  const tracker = new Tracker({ transport, queue, plugins, console });
 *
 *  See also `makeDefaultTransport`, `makeDefaultQueue` and
 *  `makeBrowserTrackerDefaultPluginList` for the actual implementation.
 *
 */
declare class BrowserTracker extends Tracker {
    // A copy of the original configuration
    readonly trackerConfig: TrackerConfig;
    constructor(trackerConfig: BrowserTrackerConfig, ...contextConfigs: ContextsConfig[]);
}
/**
 * A TrackedElement is either a TaggedElement or an EventTarget
 */
type TrackedElement = TaggableElement | EventTarget;
/**
 * Generates a location stack for the given Element. If a Tracker instance is provided, also predicts its mutations.
 *
 * 1. Traverses the DOM to reconstruct the component stack
 * 2. If a Tracker instance is provided, retrieves the Tracker's Location Stack
 * 3. Merges the two Location Stacks to reconstruct the full Location
 * 4. If a Tracker instance is provided, runs the Tracker's plugins `beforeTransport` lifecycle on the locationStack
 */
declare const getElementLocationStack: (parameters: {
    element: TrackedElement;
    tracker?: BrowserTracker;
}) => LocationStack;
/**
 * Helper function to get the current Location href
 */
declare const getLocationHref: () => string | undefined;
/**
 * Window extension for our namespace
 */
declare global {
    interface Window {
        objectiv: {
            trackers: TrackerRepository<BrowserTracker>;
        };
    }
}
/**
 * Executes `functionToRun(value)` only if the given `value` is not `undefined`
 */
declare const runIfValueIsNotUndefined: (functionToRun: Function, value: unknown) => any;
/**
 * Generic onError callback definition used by `trackerErrorHandler`.
 */
type TrackerErrorHandlerCallback = <T = unknown>(error: unknown, parameters?: T) => void;
/**
 * Generic onError callback, parameter and error handler for tag and track functions.
 * Allows developers to provide an onError callback to handle errors themselves.
 * Default behavior is to console.error.
 */
declare const trackerErrorHandler: (error: unknown, parameters?: unknown, onError?: TrackerErrorHandlerCallback | undefined) => undefined;
/**
 * Helper function to check if window exists. Used to detect non-browser environments or SSR.
 */
declare const windowExists: () => boolean;
/**
 * The parameters of `tagChildren`
 */
declare const ChildrenTaggingQueries: import("superstruct").Struct<{
    queryAll: string;
    tagAs?: {
        "data-objectiv-element-id": string;
        "data-objectiv-context": string;
        "data-objectiv-parent-element-id"?: string | undefined;
        "data-objectiv-track-clicks"?: string | undefined;
        "data-objectiv-track-blurs"?: string | undefined;
        "data-objectiv-track-visibility"?: string | undefined;
        "data-objectiv-validate"?: string | undefined;
    } | undefined;
}[], import("superstruct").Struct<{
    queryAll: string;
    tagAs?: {
        "data-objectiv-element-id": string;
        "data-objectiv-context": string;
        "data-objectiv-parent-element-id"?: string | undefined;
        "data-objectiv-track-clicks"?: string | undefined;
        "data-objectiv-track-blurs"?: string | undefined;
        "data-objectiv-track-visibility"?: string | undefined;
        "data-objectiv-validate"?: string | undefined;
    } | undefined;
}, {
    queryAll: import("superstruct").Struct<string, null>;
    tagAs: import("superstruct").Struct<{
        "data-objectiv-element-id": string;
        "data-objectiv-context": string;
        "data-objectiv-parent-element-id"?: string | undefined;
        "data-objectiv-track-clicks"?: string | undefined;
        "data-objectiv-track-blurs"?: string | undefined;
        "data-objectiv-track-visibility"?: string | undefined;
        "data-objectiv-validate"?: string | undefined;
    } | undefined, {
        "data-objectiv-element-id": import("superstruct").Struct<string, null>;
        "data-objectiv-parent-element-id": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-context": import("superstruct").Struct<string, null>;
        "data-objectiv-track-clicks": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-track-blurs": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-track-visibility": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-validate": import("superstruct").Struct<string | undefined, null>;
    }>;
}>>;
type ChildrenTaggingQueries = Infer<typeof ChildrenTaggingQueries>;
/**
 * The parameters of `tagChild`
 */
declare const ChildrenTaggingQuery: import("superstruct").Struct<{
    queryAll: string;
    tagAs?: {
        "data-objectiv-element-id": string;
        "data-objectiv-context": string;
        "data-objectiv-parent-element-id"?: string | undefined;
        "data-objectiv-track-clicks"?: string | undefined;
        "data-objectiv-track-blurs"?: string | undefined;
        "data-objectiv-track-visibility"?: string | undefined;
        "data-objectiv-validate"?: string | undefined;
    } | undefined;
}, {
    queryAll: import("superstruct").Struct<string, null>;
    tagAs: import("superstruct").Struct<{
        "data-objectiv-element-id": string;
        "data-objectiv-context": string;
        "data-objectiv-parent-element-id"?: string | undefined;
        "data-objectiv-track-clicks"?: string | undefined;
        "data-objectiv-track-blurs"?: string | undefined;
        "data-objectiv-track-visibility"?: string | undefined;
        "data-objectiv-validate"?: string | undefined;
    } | undefined, {
        "data-objectiv-element-id": import("superstruct").Struct<string, null>;
        "data-objectiv-parent-element-id": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-context": import("superstruct").Struct<string, null>;
        "data-objectiv-track-clicks": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-track-blurs": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-track-visibility": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-validate": import("superstruct").Struct<string | undefined, null>;
    }>;
}>;
type ChildrenTaggingQuery = Infer<typeof ChildrenTaggingQuery>;
/**
 * FlushQueue Options for TrackClicks TaggingAttribute
 */
declare const FlushQueueOptions: import("superstruct").Struct<boolean | "onTimeout", null>;
type FlushQueueOptions = Infer<typeof FlushQueueOptions>;
/**
 * The parameters of the Event Tracker shorthand functions
 */
type InteractiveEventTrackerParameters = {
    element: TrackedElement;
    locationStack?: LocationStack;
    globalContexts?: GlobalContexts;
    tracker?: BrowserTracker;
    onError?: TrackerErrorHandlerCallback;
};
/**
 * The object returned by `tagLocation` and its shorthands
 */
declare const TagLocationReturnValue: import("superstruct").Struct<{
    "data-objectiv-element-id": string;
    "data-objectiv-context": string;
    "data-objectiv-parent-element-id"?: string | undefined;
    "data-objectiv-track-clicks"?: string | undefined;
    "data-objectiv-track-blurs"?: string | undefined;
    "data-objectiv-track-visibility"?: string | undefined;
    "data-objectiv-validate"?: string | undefined;
} | undefined, {
    "data-objectiv-element-id": import("superstruct").Struct<string, null>;
    "data-objectiv-parent-element-id": import("superstruct").Struct<string | undefined, null>;
    "data-objectiv-context": import("superstruct").Struct<string, null>;
    "data-objectiv-track-clicks": import("superstruct").Struct<string | undefined, null>;
    "data-objectiv-track-blurs": import("superstruct").Struct<string | undefined, null>;
    "data-objectiv-track-visibility": import("superstruct").Struct<string | undefined, null>;
    "data-objectiv-validate": import("superstruct").Struct<string | undefined, null>;
}>;
type TagLocationReturnValue = Infer<typeof TagLocationReturnValue>;
/**
 * The options object that tagLocation and its shorthands accept
 */
declare const TagLocationOptions: import("superstruct").Struct<{
    trackClicks?: boolean | {
        waitUntilTracked: true | {
            intervalMs?: number | undefined;
            timeoutMs?: number | undefined;
            flushQueue?: boolean | "onTimeout" | undefined;
        };
    } | undefined;
    trackBlurs?: boolean | undefined;
    trackVisibility?: {
        mode: "auto";
    } | {
        mode: "manual";
        isVisible: boolean;
    } | undefined;
    parent?: {
        "data-objectiv-element-id": string;
        "data-objectiv-context": string;
        "data-objectiv-parent-element-id"?: string | undefined;
        "data-objectiv-track-clicks"?: string | undefined;
        "data-objectiv-track-blurs"?: string | undefined;
        "data-objectiv-track-visibility"?: string | undefined;
        "data-objectiv-validate"?: string | undefined;
    } | undefined;
    validate?: {
        locationUniqueness: boolean;
    } | undefined;
}, {
    trackClicks: import("superstruct").Struct<boolean | {
        waitUntilTracked: true | {
            intervalMs?: number | undefined;
            timeoutMs?: number | undefined;
            flushQueue?: boolean | "onTimeout" | undefined;
        };
    } | undefined, null>;
    trackBlurs: import("superstruct").Struct<boolean | undefined, null>;
    trackVisibility: import("superstruct").Struct<{
        mode: "auto";
    } | {
        mode: "manual";
        isVisible: boolean;
    } | undefined, null>;
    parent: import("superstruct").Struct<{
        "data-objectiv-element-id": string;
        "data-objectiv-context": string;
        "data-objectiv-parent-element-id"?: string | undefined;
        "data-objectiv-track-clicks"?: string | undefined;
        "data-objectiv-track-blurs"?: string | undefined;
        "data-objectiv-track-visibility"?: string | undefined;
        "data-objectiv-validate"?: string | undefined;
    } | undefined, {
        "data-objectiv-element-id": import("superstruct").Struct<string, null>;
        "data-objectiv-parent-element-id": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-context": import("superstruct").Struct<string, null>;
        "data-objectiv-track-clicks": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-track-blurs": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-track-visibility": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-validate": import("superstruct").Struct<string | undefined, null>;
    }>;
    validate: import("superstruct").Struct<{
        locationUniqueness: boolean;
    } | undefined, {
        locationUniqueness: import("superstruct").Struct<boolean, null>;
    }>;
}>;
type TagLocationOptions = {
    trackClicks?: TrackClicksAttribute;
    trackBlurs?: boolean;
    trackVisibility?: TrackVisibilityAttribute;
    parent?: TagLocationReturnValue;
    validate?: ValidateAttribute;
};
/**
 * LocationTaggers are shorthands around tagLocation.
 */
declare const LocationTaggerParameters: import("superstruct").Struct<{
    id: string;
    options?: {
        trackClicks?: boolean | {
            waitUntilTracked: true | {
                intervalMs?: number | undefined;
                timeoutMs?: number | undefined;
                flushQueue?: boolean | "onTimeout" | undefined;
            };
        } | undefined;
        trackBlurs?: boolean | undefined;
        trackVisibility?: {
            mode: "auto";
        } | {
            mode: "manual";
            isVisible: boolean;
        } | undefined;
        parent?: {
            "data-objectiv-element-id": string;
            "data-objectiv-context": string;
            "data-objectiv-parent-element-id"?: string | undefined;
            "data-objectiv-track-clicks"?: string | undefined;
            "data-objectiv-track-blurs"?: string | undefined;
            "data-objectiv-track-visibility"?: string | undefined;
            "data-objectiv-validate"?: string | undefined;
        } | undefined;
        validate?: {
            locationUniqueness: boolean;
        } | undefined;
    } | undefined;
    onError?: Function | undefined;
}, {
    id: import("superstruct").Struct<string, null>;
    options: import("superstruct").Struct<{
        trackClicks?: boolean | {
            waitUntilTracked: true | {
                intervalMs?: number | undefined;
                timeoutMs?: number | undefined;
                flushQueue?: boolean | "onTimeout" | undefined;
            };
        } | undefined;
        trackBlurs?: boolean | undefined;
        trackVisibility?: {
            mode: "auto";
        } | {
            mode: "manual";
            isVisible: boolean;
        } | undefined;
        parent?: {
            "data-objectiv-element-id": string;
            "data-objectiv-context": string;
            "data-objectiv-parent-element-id"?: string | undefined;
            "data-objectiv-track-clicks"?: string | undefined;
            "data-objectiv-track-blurs"?: string | undefined;
            "data-objectiv-track-visibility"?: string | undefined;
            "data-objectiv-validate"?: string | undefined;
        } | undefined;
        validate?: {
            locationUniqueness: boolean;
        } | undefined;
    } | undefined, {
        trackClicks: import("superstruct").Struct<boolean | {
            waitUntilTracked: true | {
                intervalMs?: number | undefined;
                timeoutMs?: number | undefined;
                flushQueue?: boolean | "onTimeout" | undefined;
            };
        } | undefined, null>;
        trackBlurs: import("superstruct").Struct<boolean | undefined, null>;
        trackVisibility: import("superstruct").Struct<{
            mode: "auto";
        } | {
            mode: "manual";
            isVisible: boolean;
        } | undefined, null>;
        parent: import("superstruct").Struct<{
            "data-objectiv-element-id": string;
            "data-objectiv-context": string;
            "data-objectiv-parent-element-id"?: string | undefined;
            "data-objectiv-track-clicks"?: string | undefined;
            "data-objectiv-track-blurs"?: string | undefined;
            "data-objectiv-track-visibility"?: string | undefined;
            "data-objectiv-validate"?: string | undefined;
        } | undefined, {
            "data-objectiv-element-id": import("superstruct").Struct<string, null>;
            "data-objectiv-parent-element-id": import("superstruct").Struct<string | undefined, null>;
            "data-objectiv-context": import("superstruct").Struct<string, null>;
            "data-objectiv-track-clicks": import("superstruct").Struct<string | undefined, null>;
            "data-objectiv-track-blurs": import("superstruct").Struct<string | undefined, null>;
            "data-objectiv-track-visibility": import("superstruct").Struct<string | undefined, null>;
            "data-objectiv-validate": import("superstruct").Struct<string | undefined, null>;
        }>;
        validate: import("superstruct").Struct<{
            locationUniqueness: boolean;
        } | undefined, {
            locationUniqueness: import("superstruct").Struct<boolean, null>;
        }>;
    }>;
    onError: import("superstruct").Struct<Function | undefined, null>;
}>;
type LocationTaggerParameters = {
    id: string;
    options?: TagLocationOptions;
    onError?: TrackerErrorHandlerCallback;
};
/**
 * The same parameters of regular Event Trackers, but all attributes are optional.
 */
type NonInteractiveEventTrackerParameters = Partial<InteractiveEventTrackerParameters>;
/**
 * The definition of the object returned by `tagChildren` and `tagChild`
 */
declare const TagChildrenReturnValue: import("superstruct").Struct<{
    "data-objectiv-tag-children": string;
} | undefined, {
    "data-objectiv-tag-children": import("superstruct").Struct<string, null>;
}>;
type TagChildrenReturnValue = Infer<typeof TagChildrenReturnValue>;
/**
 * tagLink has one extra attribute, `href`, as mandatory parameter.
 */
declare const TagLinkParameters: import("superstruct").Struct<{
    id: string;
    href: string;
    options?: {
        trackClicks?: boolean | {
            waitUntilTracked: true | {
                intervalMs?: number | undefined;
                timeoutMs?: number | undefined;
                flushQueue?: boolean | "onTimeout" | undefined;
            };
        } | undefined;
        trackBlurs?: boolean | undefined;
        trackVisibility?: {
            mode: "auto";
        } | {
            mode: "manual";
            isVisible: boolean;
        } | undefined;
        parent?: {
            "data-objectiv-element-id": string;
            "data-objectiv-context": string;
            "data-objectiv-parent-element-id"?: string | undefined;
            "data-objectiv-track-clicks"?: string | undefined;
            "data-objectiv-track-blurs"?: string | undefined;
            "data-objectiv-track-visibility"?: string | undefined;
            "data-objectiv-validate"?: string | undefined;
        } | undefined;
        validate?: {
            locationUniqueness: boolean;
        } | undefined;
    } | undefined;
    onError?: Function | undefined;
}, {
    href: import("superstruct").Struct<string, null>;
    id: import("superstruct").Struct<string, null>;
    options: import("superstruct").Struct<{
        trackClicks?: boolean | {
            waitUntilTracked: true | {
                intervalMs?: number | undefined;
                timeoutMs?: number | undefined;
                flushQueue?: boolean | "onTimeout" | undefined;
            };
        } | undefined;
        trackBlurs?: boolean | undefined;
        trackVisibility?: {
            mode: "auto";
        } | {
            mode: "manual";
            isVisible: boolean;
        } | undefined;
        parent?: {
            "data-objectiv-element-id": string;
            "data-objectiv-context": string;
            "data-objectiv-parent-element-id"?: string | undefined;
            "data-objectiv-track-clicks"?: string | undefined;
            "data-objectiv-track-blurs"?: string | undefined;
            "data-objectiv-track-visibility"?: string | undefined;
            "data-objectiv-validate"?: string | undefined;
        } | undefined;
        validate?: {
            locationUniqueness: boolean;
        } | undefined;
    } | undefined, {
        trackClicks: import("superstruct").Struct<boolean | {
            waitUntilTracked: true | {
                intervalMs?: number | undefined;
                timeoutMs?: number | undefined;
                flushQueue?: boolean | "onTimeout" | undefined;
            };
        } | undefined, null>;
        trackBlurs: import("superstruct").Struct<boolean | undefined, null>;
        trackVisibility: import("superstruct").Struct<{
            mode: "auto";
        } | {
            mode: "manual";
            isVisible: boolean;
        } | undefined, null>;
        parent: import("superstruct").Struct<{
            "data-objectiv-element-id": string;
            "data-objectiv-context": string;
            "data-objectiv-parent-element-id"?: string | undefined;
            "data-objectiv-track-clicks"?: string | undefined;
            "data-objectiv-track-blurs"?: string | undefined;
            "data-objectiv-track-visibility"?: string | undefined;
            "data-objectiv-validate"?: string | undefined;
        } | undefined, {
            "data-objectiv-element-id": import("superstruct").Struct<string, null>;
            "data-objectiv-parent-element-id": import("superstruct").Struct<string | undefined, null>;
            "data-objectiv-context": import("superstruct").Struct<string, null>;
            "data-objectiv-track-clicks": import("superstruct").Struct<string | undefined, null>;
            "data-objectiv-track-blurs": import("superstruct").Struct<string | undefined, null>;
            "data-objectiv-track-visibility": import("superstruct").Struct<string | undefined, null>;
            "data-objectiv-validate": import("superstruct").Struct<string | undefined, null>;
        }>;
        validate: import("superstruct").Struct<{
            locationUniqueness: boolean;
        } | undefined, {
            locationUniqueness: import("superstruct").Struct<boolean, null>;
        }>;
    }>;
    onError: import("superstruct").Struct<Function | undefined, null>;
}>;
type TagLinkParameters = LocationTaggerParameters & {
    href: string;
};
/**
 * The parameters of `tagLocation` and its shorthands
 */
declare const TagLocationParameters: import("superstruct").Struct<{
    instance: {
        _type: "RootLocationContext";
        id: string;
        __location_context: true;
    } | {
        _type: "ContentContext";
        id: string;
        __location_context: true;
    } | {
        _type: "ExpandableContext";
        id: string;
        __location_context: true;
    } | {
        _type: "MediaPlayerContext";
        id: string;
        __location_context: true;
    } | {
        _type: "NavigationContext";
        id: string;
        __location_context: true;
    } | {
        _type: "OverlayContext";
        id: string;
        __location_context: true;
    } | {
        _type: "InputContext";
        id: string;
        __location_context: true;
    } | {
        _type: "PressableContext";
        id: string;
        __location_context: true;
        __pressable_context: true;
    } | {
        _type: "LinkContext";
        id: string;
        __location_context: true;
        __pressable_context: true;
        href: string;
    };
    options?: {
        trackClicks?: boolean | {
            waitUntilTracked: true | {
                intervalMs?: number | undefined;
                timeoutMs?: number | undefined;
                flushQueue?: boolean | "onTimeout" | undefined;
            };
        } | undefined;
        trackBlurs?: boolean | undefined;
        trackVisibility?: {
            mode: "auto";
        } | {
            mode: "manual";
            isVisible: boolean;
        } | undefined;
        parent?: {
            "data-objectiv-element-id": string;
            "data-objectiv-context": string;
            "data-objectiv-parent-element-id"?: string | undefined;
            "data-objectiv-track-clicks"?: string | undefined;
            "data-objectiv-track-blurs"?: string | undefined;
            "data-objectiv-track-visibility"?: string | undefined;
            "data-objectiv-validate"?: string | undefined;
        } | undefined;
        validate?: {
            locationUniqueness: boolean;
        } | undefined;
    } | undefined;
    onError?: Function | undefined;
}, {
    instance: import("superstruct").Struct<{
        _type: "RootLocationContext";
        id: string;
        __location_context: true;
    } | {
        _type: "ContentContext";
        id: string;
        __location_context: true;
    } | {
        _type: "ExpandableContext";
        id: string;
        __location_context: true;
    } | {
        _type: "MediaPlayerContext";
        id: string;
        __location_context: true;
    } | {
        _type: "NavigationContext";
        id: string;
        __location_context: true;
    } | {
        _type: "OverlayContext";
        id: string;
        __location_context: true;
    } | {
        _type: "InputContext";
        id: string;
        __location_context: true;
    } | {
        _type: "PressableContext";
        id: string;
        __location_context: true;
        __pressable_context: true;
    } | {
        _type: "LinkContext";
        id: string;
        __location_context: true;
        __pressable_context: true;
        href: string;
    }, null>;
    options: import("superstruct").Struct<{
        trackClicks?: boolean | {
            waitUntilTracked: true | {
                intervalMs?: number | undefined;
                timeoutMs?: number | undefined;
                flushQueue?: boolean | "onTimeout" | undefined;
            };
        } | undefined;
        trackBlurs?: boolean | undefined;
        trackVisibility?: {
            mode: "auto";
        } | {
            mode: "manual";
            isVisible: boolean;
        } | undefined;
        parent?: {
            "data-objectiv-element-id": string;
            "data-objectiv-context": string;
            "data-objectiv-parent-element-id"?: string | undefined;
            "data-objectiv-track-clicks"?: string | undefined;
            "data-objectiv-track-blurs"?: string | undefined;
            "data-objectiv-track-visibility"?: string | undefined;
            "data-objectiv-validate"?: string | undefined;
        } | undefined;
        validate?: {
            locationUniqueness: boolean;
        } | undefined;
    } | undefined, {
        trackClicks: import("superstruct").Struct<boolean | {
            waitUntilTracked: true | {
                intervalMs?: number | undefined;
                timeoutMs?: number | undefined;
                flushQueue?: boolean | "onTimeout" | undefined;
            };
        } | undefined, null>;
        trackBlurs: import("superstruct").Struct<boolean | undefined, null>;
        trackVisibility: import("superstruct").Struct<{
            mode: "auto";
        } | {
            mode: "manual";
            isVisible: boolean;
        } | undefined, null>;
        parent: import("superstruct").Struct<{
            "data-objectiv-element-id": string;
            "data-objectiv-context": string;
            "data-objectiv-parent-element-id"?: string | undefined;
            "data-objectiv-track-clicks"?: string | undefined;
            "data-objectiv-track-blurs"?: string | undefined;
            "data-objectiv-track-visibility"?: string | undefined;
            "data-objectiv-validate"?: string | undefined;
        } | undefined, {
            "data-objectiv-element-id": import("superstruct").Struct<string, null>;
            "data-objectiv-parent-element-id": import("superstruct").Struct<string | undefined, null>;
            "data-objectiv-context": import("superstruct").Struct<string, null>;
            "data-objectiv-track-clicks": import("superstruct").Struct<string | undefined, null>;
            "data-objectiv-track-blurs": import("superstruct").Struct<string | undefined, null>;
            "data-objectiv-track-visibility": import("superstruct").Struct<string | undefined, null>;
            "data-objectiv-validate": import("superstruct").Struct<string | undefined, null>;
        }>;
        validate: import("superstruct").Struct<{
            locationUniqueness: boolean;
        } | undefined, {
            locationUniqueness: import("superstruct").Struct<boolean, null>;
        }>;
    }>;
    onError: import("superstruct").Struct<Function | undefined, null>;
}>;
type TagLocationParameters = {
    instance: AnyLocationContext;
    options?: TagLocationOptions;
    onError?: TrackerErrorHandlerCallback;
};
/**
 * trackFailureEvent has an extra attribute, `message`, as mandatory parameter.
 */
type TrackFailureEventParameters = NonInteractiveEventTrackerParameters & {
    message: string;
};
/**
 * trackSuccessEvent has an extra attribute, `message`, as mandatory parameter.
 */
type TrackSuccessEventParameters = NonInteractiveEventTrackerParameters & {
    message: string;
};
/**
 * Struct definition for UUIDs
 */
declare const Uuid: import("superstruct").Struct<string, null>;
/**
 * The parameters of `tagChild` where `tagAs` is a valid TagLocationAttributes
 */
declare const ValidChildrenTaggingQuery: import("superstruct").Struct<{
    queryAll: string;
    tagAs: {
        "data-objectiv-element-id": string;
        "data-objectiv-context": string;
        "data-objectiv-parent-element-id"?: string | undefined;
        "data-objectiv-track-clicks"?: string | undefined;
        "data-objectiv-track-blurs"?: string | undefined;
        "data-objectiv-track-visibility"?: string | undefined;
        "data-objectiv-validate"?: string | undefined;
    };
}, {
    queryAll: import("superstruct").Struct<string, null>;
    tagAs: import("superstruct").Struct<{
        "data-objectiv-element-id": string;
        "data-objectiv-context": string;
        "data-objectiv-parent-element-id"?: string | undefined;
        "data-objectiv-track-clicks"?: string | undefined;
        "data-objectiv-track-blurs"?: string | undefined;
        "data-objectiv-track-visibility"?: string | undefined;
        "data-objectiv-validate"?: string | undefined;
    }, {
        "data-objectiv-element-id": import("superstruct").Struct<string, null>;
        "data-objectiv-parent-element-id": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-context": import("superstruct").Struct<string, null>;
        "data-objectiv-track-clicks": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-track-blurs": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-track-visibility": import("superstruct").Struct<string | undefined, null>;
        "data-objectiv-validate": import("superstruct").Struct<string | undefined, null>;
    }>;
}>;
type ValidChildrenTaggingQuery = Infer<typeof ValidChildrenTaggingQuery>;
/**
 * WaitForQueueOptions Options for TrackClicks TaggingAttribute
 */
declare const WaitForQueueOptions: import("superstruct").Struct<{
    intervalMs?: number | undefined;
    timeoutMs?: number | undefined;
}, null>;
type WaitForQueueOptions = Infer<typeof WaitForQueueOptions>;
/**
 * WaitUntilTracked Options for TrackClick TaggingAttribute
 */
declare const WaitUntilTrackedOptions: import("superstruct").Struct<{
    intervalMs?: number | undefined;
    timeoutMs?: number | undefined;
    flushQueue?: boolean | "onTimeout" | undefined;
}, {
    intervalMs: import("superstruct").Struct<number | undefined, null>;
    timeoutMs: import("superstruct").Struct<number | undefined, null>;
    flushQueue: import("superstruct").Struct<boolean | "onTimeout" | undefined, null>;
}>;
type WaitUntilTrackedOptions = Infer<typeof WaitUntilTrackedOptions>;
/**
 * trackFailureEvent is a shorthand for trackEvent. It eases triggering FailureEvent programmatically.
 */
declare const trackFailureEvent: (parameters: TrackFailureEventParameters) => void;
/**
 * trackApplicationLoadedEvent is a shorthand for trackEvent. It eases triggering ApplicationLoadedEvent
 * programmatically.
 */
declare const trackApplicationLoadedEvent: (parameters?: NonInteractiveEventTrackerParameters) => void;
/**
 * trackPressEvent is a shorthand for trackEvent. It eases triggering PressEvent programmatically.
 */
declare const trackPressEvent: ({ element, locationStack, globalContexts, tracker, onError }: InteractiveEventTrackerParameters) => void;
/**
 * trackSuccessEvent is a shorthand for trackEvent. It eases triggering SuccessEvent programmatically.
 */
declare const trackSuccessEvent: (parameters: TrackSuccessEventParameters) => void;
/**
 * 1. Reconstruct a LocationStack for the given Element by traversing its DOM parents
 * 2. Factors a new Event with the given `_type`
 * 3. Tracks the new Event via WebTracker
 */
declare const trackEvent: (parameters: {
    event: UntrackedEvent;
    element?: TrackedElement;
    tracker?: BrowserTracker;
    trackerId?: string;
    onError?: TrackerErrorHandlerCallback;
}) => void;
/**
 * trackInputChangeEvent is a shorthand for trackEvent. It eases triggering InputChangeEvent programmatically.
 */
declare const trackInputChangeEvent: ({ element, locationStack, globalContexts, tracker, onError }: InteractiveEventTrackerParameters) => void;
/**
 * trackInteractiveEvent is a shorthand for trackEvent. It eases triggering InteractiveEvent programmatically.
 */
declare const trackInteractiveEvent: ({ element, locationStack, globalContexts, tracker, onError }: InteractiveEventTrackerParameters) => void;
/**
 * trackHiddenEvent is a shorthand for trackEvent. It eases triggering HiddenEvent programmatically.
 */
declare const trackHiddenEvent: ({ element, locationStack, globalContexts, tracker, onError }: InteractiveEventTrackerParameters) => void;
/**
 * trackVisibleEvent is a shorthand for trackEvent. It eases triggering VisibleEvent programmatically.
 */
declare const trackVisibleEvent: ({ element, locationStack, globalContexts, tracker, onError }: InteractiveEventTrackerParameters) => void;
/**
 * trackMediaEvent is a shorthand for trackEvent. It eases triggering MediaEvent programmatically.
 */
declare const trackMediaEvent: ({ element, locationStack, globalContexts, tracker, onError }: InteractiveEventTrackerParameters) => void;
/**
 * trackMediaLoadEvent is a shorthand for trackEvent. It eases triggering MediaLoadEvent programmatically.
 */
declare const trackMediaLoadEvent: ({ element, locationStack, globalContexts, tracker, onError }: InteractiveEventTrackerParameters) => void;
/**
 * trackMediaPauseEvent is a shorthand for trackEvent. It eases triggering MediaPauseEvent programmatically.
 */
declare const trackMediaPauseEvent: ({ element, locationStack, globalContexts, tracker, onError }: InteractiveEventTrackerParameters) => void;
/**
 * trackMediaStartEvent is a shorthand for trackEvent. It eases triggering MediaStartEvent programmatically.
 */
declare const trackMediaStartEvent: ({ element, locationStack, globalContexts, tracker, onError }: InteractiveEventTrackerParameters) => void;
/**
 * trackMediaStopEvent is a shorthand for trackEvent. It eases triggering MediaStopEvent programmatically.
 */
declare const trackMediaStopEvent: ({ element, locationStack, globalContexts, tracker, onError }: InteractiveEventTrackerParameters) => void;
/**
 * trackNonInteractiveEvent is a shorthand for trackEvent. It eases triggering NonInteractiveEvent programmatically.
 */
declare const trackNonInteractiveEvent: ({ element, locationStack, globalContexts, tracker, onError }: NonInteractiveEventTrackerParameters) => void;
/**
 * trackVisibility is a shorthand for trackEvent. It eases triggering visibility events programmatically
 */
declare const trackVisibility: ({ element, locationStack, globalContexts, tracker, isVisible, onError }: InteractiveEventTrackerParameters & {
    isVisible: boolean;
}) => void;
/**
 * Syntactic sugar to track only one child.
 *
 * Examples
 *
 *    tagChild({
 *      query: '#button1',
 *      tagAs: tagPressable({ id: 'button1', text: 'Button 1' })
 *    })
 *
 *    tagChild({
 *      query: '#button2',
 *      tagAs: tagPressable({ id: 'button2', text: 'Button 2' })
 *    })
 *
 */
declare const tagChild: (parameters: ChildrenTaggingQuery, onError?: TrackerErrorHandlerCallback | undefined) => TagChildrenReturnValue;
/**
 * Used to decorate a TaggableElement with our Children Tagging Attributes.
 *
 * Returns an object containing the Children Tagging Attribute only.
 * This attribute is a serialized list of ChildrenTaggingQuery objects and will be parsed and executed by our Observer
 * as soon as the elements gets rendered.
 *
 * Examples
 *
 *    tagChildren([
 *      {
 *        queryAll: 'button[aria-label="Previous"]',
 *        tagAs: tagPressable({ id: 'prev', text: 'Previous' })
 *      },
 *      {
 *        queryAll: 'button[aria-label="Next"]',
 *        tagAs: tagPressable({ id: 'next', text: 'Next' })
 *      }
 *    ])
 */
declare const tagChildren: (parameters: {
    queryAll: string;
    tagAs?: {
        "data-objectiv-element-id": string;
        "data-objectiv-context": string;
        "data-objectiv-parent-element-id"?: string | undefined;
        "data-objectiv-track-clicks"?: string | undefined;
        "data-objectiv-track-blurs"?: string | undefined;
        "data-objectiv-track-visibility"?: string | undefined;
        "data-objectiv-validate"?: string | undefined;
    } | undefined;
}[], onError?: TrackerErrorHandlerCallback | undefined) => TagChildrenReturnValue;
/**
 * tagContent is a shorthand for tagLocation. It eases the tagging of ContentContext bound Elements.
 */
declare const tagContent: (parameters: LocationTaggerParameters) => TagLocationReturnValue;
/**
 * tagExpandable is a shorthand for tagLocation. It eases the tagging of ExpandableContext bound Elements
 */
declare const tagExpandable: (parameters: LocationTaggerParameters) => TagLocationReturnValue;
/**
 * tagInput is a shorthand for tagLocation. It eases the tagging of InputContext bound Elements
 */
declare const tagInput: (parameters: LocationTaggerParameters) => TagLocationReturnValue;
/**
 * tagLink is a shorthand for tagLocation. It eases the tagging of LinkContext bound Elements
 */
declare const tagLink: (parameters: TagLinkParameters) => TagLocationReturnValue;
/**
 * Used to decorate a Taggable Element with our Tagging Attributes.
 *
 * Returns an object containing the Tagging Attributes. It's properties are supposed to be spread on the target HTML
 * Element. This allows us to identify elements uniquely in a Document and to reconstruct their Location.
 *
 * For a higher level api see the tagLocationHelpers module.
 *
 * Examples
 *
 *    tagLocation({ instance: makeElementContext({ id: 'section-id' }) })
 *    tagLocation({ instance: makeElementContext({ id: 'section-id' }), { trackClicks: true } })
 *
 */
declare const tagLocation: (parameters: TagLocationParameters) => TagLocationReturnValue;
/**
 * tagMediaPlayer is a shorthand for tagLocation. It eases the tagging of MediaPlayerContext bound Elements
 */
declare const tagMediaPlayer: (parameters: LocationTaggerParameters) => TagLocationReturnValue;
/**
 * tagNavigation is a shorthand for tagLocation. It eases the tagging of NavigationContext bound Elements
 */
declare const tagNavigation: (parameters: LocationTaggerParameters) => TagLocationReturnValue;
/**
 * tagOverlay is a shorthand for tagLocation. It eases the tagging of OverlayContext bound Elements
 */
declare const tagOverlay: (parameters: LocationTaggerParameters) => TagLocationReturnValue;
/**
 * tagPressable is a shorthand for tagLocation. It eases the tagging of ButtonContext bound Elements
 */
declare const tagPressable: (parameters: LocationTaggerParameters) => TagLocationReturnValue;
/**
 * tagRootLocation is a shorthand for tagLocation. It eases the tagging of RootLocationContext bound Elements.
 */
declare const tagRootLocation: (parameters: LocationTaggerParameters) => TagLocationReturnValue;
/**
 * Hold an instance of the Mutation Observer and some state for some global events, like URLs and Application Loaded.
 */
declare const AutoTrackingState: {
    observerInstance: MutationObserver | null;
    applicationLoaded: boolean;
    previousURL: string | undefined;
};
/**
 * A factory to make the event handler to attach to new TaggedElements with the `trackBlurs` attributes set
 */
declare const makeBlurEventHandler: (element: TaggedElement, tracker?: BrowserTracker | undefined) => (event: Event) => void;
/**
 * A factory to make the event handler to attach to new TaggedElements with the `trackClicks` attributes set
 */
declare const makeClickEventHandler: (element: TaggedElement, tracker: BrowserTracker, trackClicksOptions?: TrackClicksOptions) => (event: Event) => Promise<void>;
/**
 * A factory to generate our mutation observer callback. It will observe:
 *
 * New DOM nodes added.
 * We use a Mutation Observer to monitor the DOM for subtrees being added.
 * When that happens we traverse the new Nodes and scout for Elements that have been enriched with our Tagging
 * Attributes. For those Elements we attach Event listeners which will automatically handle their tracking.
 * New Elements are also added to TrackerElementLocations and their Location Stack is validated for uniqueness.
 *
 * Existing nodes changing.
 * The same Observer is also configured to monitor changes in our visibility and element id attributes.
 * When we detect a change in the visibility of a tagged element we trigger the corresponding visibility events.
 * Element id changes are used to keep the TrackerElementLocations in sync with the DOM.
 *
 * Existing nodes being removed.
 * We also monitor nodes that are removed. If those nodes are Tagged Elements of which we were tracking visibility
 * we will trigger visibility: hidden events for them.
 * We also clean them up from TrackerElementLocations.
 */
declare const makeMutationCallback: (console?: TrackerConsole | undefined) => MutationCallback;
/**
 * Check if Element is a ChildrenTaggingElement. If so:
 * - Run its children tracking queries
 * - Decorate matching Elements into TaggedElements
 * - Return a list of the decorated Elements
 */
declare const processTagChildrenElement: (element: Element) => TaggedElement[];
/**
 * Attaches event handlers to the given Element and triggers visibility Events for it if the Tagging Attributes allow.
 * - All Elements will be checked for visibility tracking and appropriate events will be triggered for them.
 * - Elements with the Objectiv Track Click attribute are bound to EventListener for Buttons, Links.
 * - Elements with the Objectiv Track Blur attribute are bound to EventListener for Inputs.
 * - All processed Elements are decorated with the `tracked` Tagging Attribute so we won't process them again.
 */
declare const trackNewElement: (element: Element, tracker: BrowserTracker, console?: TrackerConsole | undefined) => void;
/**
 * Given a Mutation Observer node containing newly added nodes it will track visibility and attach events to them:
 */
declare const trackNewElements: (element: Element, tracker: BrowserTracker, console?: TrackerConsole | undefined) => void;
/**
 * Given a removed GuardableElement node it will:
 *
 *   1. Determine whether to track a visibility:hidden event for it.
 *      Hidden Events are triggered only for automatically tracked Elements.
 *
 *   2. Remove the Element from the TrackerElementLocations state.
 *      This is both a clean-up and a way to allow it to be re-rendered as-is, as it happens with some UI libraries.
 */
declare const trackRemovedElement: (element: GuardableElement, tracker: BrowserTracker) => void;
/**
 * Invokes `trackRemovedElement` for given node and all of its children if they have the `elementId` Tagging Attribute.
 */
declare const trackRemovedElements: (element: Element, tracker: BrowserTracker) => void;
/**
 * Checks whether to trigger a visibility: hidden event for the given TaggedElement.
 * Hidden Events are triggered only for Elements that have their visibility manually set to not visible.
 */
declare const trackVisibilityHiddenEvent: (element: TaggedElement, tracker: BrowserTracker) => void;
/**
 * Checks whether to trigger a visibility: visible event for the given TaggedElement.
 * Visible Events are triggered only for Elements that have their visibility auto-tracked or manually set to visible.
 */
declare const trackVisibilityVisibleEvent: (element: TaggedElement, tracker: BrowserTracker) => void;
/**
 * Retrieves a specific Tracker's instance from the TrackerRepository or creates one if it doesn't exist yet.
 * Throws if the given Tracker ID exists but its configuration doesn't match the given one.
 */
declare const getOrMakeTracker: (trackerConfig: BrowserTrackerConfig) => BrowserTracker;
/**
 * Retrieves a specific instance of the tracker from the TrackerRepository.
 */
declare const getTracker: (trackerId?: string | undefined) => BrowserTracker;
/**
 * Retrieves the TrackerRepository instance from the window.objectiv global namespace
 */
declare const getTrackerRepository: () => TrackerRepository<BrowserTracker>;
/**
 * Allows to easily create and configure a new BrowserTracker instance and also starts auto tracking
 */
declare const makeTracker: (trackerConfig: BrowserTrackerConfig) => BrowserTracker;
/**
 * Helper method to easily set a different default Tracker in the TrackerRepository.
 */
declare const setDefaultTracker: (parameters: string | {
    trackerId: string;
    waitForQueue?: false | {
        intervalMs?: number | undefined;
        timeoutMs?: number | undefined;
    } | undefined;
    flushQueue?: boolean | "onTimeout" | undefined;
}) => Promise<void>;
/**
 * Initializes our automatic tracking, based on Mutation Observer.
 * Also tracks application Loaded.
 * Safe to call multiple times: it will auto-track only once.
 */
declare const startAutoTracking: (options?: Pick<BrowserTrackerConfig, "console" | "trackApplicationLoadedEvent"> | undefined) => void;
/**
 * Stops autoTracking
 */
declare const stopAutoTracking: () => void;
export { ApplicationContextPluginConfig, ApplicationContextPlugin, cleanObjectFromInternalProperties, LocationStack, GlobalContexts, ContextsConfig, makeApplicationContext, makeContentContext, makeCookieIdContext, makeExpandableContext, makeHttpContext, makeInputContext, makeLinkContext, makeMediaPlayerContext, makeNavigationContext, makeOverlayContext, makePathContext, makePressableContext, makeRootLocationContext, makeSessionContext, makeApplicationLoadedEvent, makeFailureEvent, makeHiddenEvent, makeInputChangeEvent, makeInteractiveEvent, makeMediaEvent, makeMediaLoadEvent, makeMediaPauseEvent, makeMediaStartEvent, makeMediaStopEvent, makeNonInteractiveEvent, makePressEvent, makeSuccessEvent, makeVisibleEvent, getObjectKeys, NonEmptyArray, isNonEmptyArray, generateUUID, waitForPromise, TrackerConfig, makeTrackerDefaultPluginsList, WaitForQueueParameters, TrackEventOptions, Tracker, TrackerConsole, NoopConsole, getLocationPath, LocationCollision, TrackerElementLocations, TrackerEventConfig, UntrackedEvent, TrackerEvent, TrackerPluginConfig, TrackerPluginInterface, TrackerPluginConstructor, TrackerPluginLifecycleInterface, TrackerPluginsConfiguration, TrackerPlugins, TrackerQueueConfig, TrackerQueue, TrackerQueueProcessFunction, TrackerQueueInterface, TrackerQueueMemoryStore, TrackerQueueStoreConfig, TrackerQueueStoreInterface, TrackerRepository, TrackerRepositoryInterface, TransportGroupConfig, TrackerTransportGroup, TransportableEvent, TransportSendError, TrackerTransportConfig, TrackerTransportInterface, TrackerTransportRetryConfig, TrackerTransportRetry, TrackerTransportRetryAttempt, TrackerTransportSwitchConfig, TrackerTransportSwitch, makeDefaultPluginsList, makeDefaultQueue, makeDefaultTransport, isParentTaggedElement, isTagChildrenElement, isTaggableElement, isTaggedElement, parseTagChildren, parseJson, parseLocationContext, parseTrackClicks, parseTrackVisibility, parseValidate, stringifyTagChildren, stringifyJson, stringifyLocationContext, stringifyTrackClicks, stringifyTrackVisibility, stringifyValidate, compareTrackerConfigs, findParentTaggedElements, getElementLocationStack, getLocationHref, runIfValueIsNotUndefined, trackerErrorHandler, windowExists, BrowserTrackerConfig, ChildrenTaggingQueries, ChildrenTaggingQuery, FlushQueueOptions, GuardableElement, InteractiveEventTrackerParameters, AbstractContext, AbstractLocationContext$0 as AbstractLocationContext, AbstractPressableContext, RootLocationContext$0 as RootLocationContext, ContentContext$0 as ContentContext, ExpandableContext$0 as ExpandableContext, MediaPlayerContext$0 as MediaPlayerContext, NavigationContext$0 as NavigationContext, OverlayContext$0 as OverlayContext, InputContext$0 as InputContext, PressableContext$0 as PressableContext, LinkContext$0 as LinkContext, AnyLocationContext, AnyPressableContext, AnyClickableContext, AnyShowableContext, LocationTaggerParameters, NonInteractiveEventTrackerParameters, ParentTaggedElement, TagLocationAttributes, TagChildrenAttributes, TagChildrenElement, TagChildrenReturnValue, TaggableElement, TaggedElement, TaggingAttribute, TagLinkParameters, TagLocationOptions, TagLocationParameters, TagLocationReturnValue, TrackClicksAttribute, TrackClicksOptions, TrackedElement, TrackerErrorHandlerCallback, TrackFailureEventParameters, TrackSuccessEventParameters, TrackVisibilityAttribute, Uuid, ValidateAttribute, ValidChildrenTaggingQuery, WaitForQueueOptions, WaitUntilTrackedOptions, trackFailureEvent, trackApplicationLoadedEvent, trackPressEvent, trackSuccessEvent, trackEvent, trackInputChangeEvent, trackInteractiveEvent, trackHiddenEvent, trackVisibleEvent, trackMediaEvent, trackMediaLoadEvent, trackMediaPauseEvent, trackMediaStartEvent, trackMediaStopEvent, trackNonInteractiveEvent, trackVisibility, tagChild, tagChildren, tagContent, tagExpandable, tagInput, tagLink, tagLocation, tagMediaPlayer, tagNavigation, tagOverlay, tagPressable, tagRootLocation, AutoTrackingState, makeBlurEventHandler, makeClickEventHandler, makeMutationCallback, processTagChildrenElement, trackNewElement, trackNewElements, trackRemovedElement, trackRemovedElements, trackVisibilityHiddenEvent, trackVisibilityVisibleEvent, BrowserTracker, getOrMakeTracker, getTracker, getTrackerRepository, makeTracker, setDefaultTracker, startAutoTracking, stopAutoTracking };
